@@ -1,114 +1,10 @@
 /*
-                 YIFF Client side to Server Network Handling
-
-	Functions:
-
-	void YNetPrintError(
-                FILE *stream,
-                YConnection *con,
-                u_int16_t chunk_length,
-                u_int16_t major_op_code,
-                u_int16_t minor_op_code,
-		char *mesg
-        )
-	void YSetEventToBeDisconnect(YEvent *event)
-	int YSendData(int s, char *buf, int len)
-
-	int YNetSendAudioChangePreset(
-		YConnection *con,
-		char *audio_mode_name
-	)
-	int YNetSendAudioChangeValues(
-		YConnection *con,
-	        int sample_size,
-	        int channels,
-	        YDataLength sample_rate,
-	        int direction, 
-	        int allow_fragmenting,
-	        int num_fragments,
-	        int fragment_size
-	)
-	int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendCycleChange(YConnection *con, long cycle_us)
-	int YNetParseCycleChange(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendDisconnect(YConnection *con, int reason)
-	int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSetHost(
-	        YConnection *con,
-	        u_int16_t minor_op_code,
-	        YIPUnion *ip
-	)
-	int YNetParseSetHost(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSetMixerChannel(
-	        YConnection *con,
-	        int mixer_code,
-	        Coefficient value1,
-	        Coefficient value2
-	)
-	int YNetSendGetMixerChannel(YConnection *con, int mixer_code)
-	int YNetParseMixerChannel(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSoundPlay(
-	        YConnection *con,
-		YID yid,
-	        char *path,     
-	        YDataPosition pos,
-	        YVolumeStruct *volume,
-	        int repeats
-	)
-	int YNetParseSoundPlay(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSoundKill(YConnection *con, YID yid)
-	int YNetParseSoundKill(YCNP_STD_INPUTS_PROTO)
-
-        int YNetSendGetSoundObjectAttributes(
-		YConnection *con,
-		char *path
-	)
-        int YNetParseSoundObjectAttributes(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendShutdown(YConnection *con, int reason)
-	int YNetParseShutdown(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSync(YConnection *con, long cycle_ahead_us)
-	int YNetParseSync(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendGetAudioStats(YConnection *con)
-        int YNetParseAudioStats(YCNP_STD_INPUTS_PROTO)
-
-        int YNetSendGetServerStats(YConnection *con)
-        int YNetParseServerStats(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendListAudioModes(YConnection *con)
-	int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO)
-
-
-	int YNetParse(
-	        YConnection *con,
-	        YEvent *event, 
-	        u_int8_t *buf,  
-	        u_int32_t chunk_length,
-	        u_int16_t major_op_code,
-	        u_int16_t minor_op_code
-	)
-
-	int YNetRecv(YConnection *con)
-
-	---
-
 	The functions contained here should only be called by functions
-	from yclientio.c!
-
+	from functions in original library source file yclientio.c
  */
 
 #include <stdio.h>
-#include <db.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <string.h>
 #include <errno.h>
 extern int h_errno;             /* For netdb.h */
@@ -116,6 +12,7 @@ extern int h_errno;             /* For netdb.h */
 #include <sys/time.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <math.h>
                 
 #define _USE_BSD
 #include <sys/types.h>
@@ -130,27 +27,115 @@ extern int h_errno;             /* For netdb.h */
 #include <arpa/inet.h>          /* For inet_ntoa() */
 #include <fcntl.h>
 
-#include "../include/Y.h"
-#include "../include/Ylib.h"
-#include "../include/Yclientnet.h"
+#include "../include/Y2/Y.h"
+#include "../include/Y2/Ylib.h"
+#include "../include/Y2/Yclientnet.h"
+
+
+void YNetPrintError(
+	FILE *stream,
+	YConnection *con,
+	u_int16_t chunk_length,
+	u_int16_t major_op_code,
+	u_int16_t minor_op_code,
+	const char *mesg
+);
+void YSetEventToBeDisconnect(YEvent *event);
+int YSendData(int s, const char *buf, int len);
+
+int YNetSendAudioChangePreset(
+	YConnection *con,
+	const char *audio_mode_name
+);
+int YNetSendAudioChangeValues(
+	YConnection *con,
+	int sample_size, 
+        int channels,
+        YDataLength sample_rate,
+        int direction,
+        int allow_fragmenting,
+        int num_fragments,
+        int fragment_size
+);
+int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendCycleChange(YConnection *con, long cycle_us);
+int YNetParseCycleChange(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendDisconnect(YConnection *con, int reason);
+int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSetHost(
+	YConnection *con,
+	u_int16_t minor_op_code,
+	YIPUnion *ip
+);
+int YNetParseSetHost(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSetMixerChannel(
+	YConnection *con,
+	int mixer_code,
+	Coefficient value1, Coefficient value2
+);
+int YNetSendGetMixerChannel(YConnection *con, int mixer_code);
+int YNetParseMixerChannel(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSoundPlay(
+	YConnection *con,
+	YID yid,
+	const char *path,
+	YDataPosition pos,
+	YVolumeStruct *volume,
+	int sample_rate,
+	int repeats
+);
+int YNetParseSoundPlay(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSoundKill(YConnection *con, YID yid);
+int YNetParseSoundKill(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendGetSoundObjectAttributes(
+	YConnection *con,
+	const char *path
+);
+int YNetParseSoundObjectAttributes(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendShutdown(YConnection *con, int reason);
+int YNetParseShutdown(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSync(YConnection *con, long cycle_ahead_us);
+int YNetParseSync(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendGetAudioStats(YConnection *con);
+int YNetParseAudioStats(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendGetServerStats(YConnection *con);
+int YNetParseServerStats(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendListAudioModes(YConnection *con);
+int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSetSoundObjectPlayValues(
+	YConnection *con,
+	YEventSoundPlay *value
+);
+int YNetParseSetSoundObjectPlayValues(YCNP_STD_INPUTS_PROTO);
+
+int YNetParse(
+	YConnection *con,
+	YEvent *event,
+	const u_int8_t *buf,
+	u_int32_t chunk_length,
+	u_int16_t major_op_code,
+	u_int16_t minor_op_code
+);
+int YNetRecv(YConnection *con);
 
 
 #define MIN(a,b)        ((a) < (b) ? (a) : (b))
 #define MAX(a,b)        ((a) > (b) ? (a) : (b))
 
 
-
-void YNetPrintError(
-	FILE *stream, YConnection *con,    
-        u_int16_t chunk_length,
-	u_int16_t major_op_code, u_int16_t minor_op_code,
-        char *mesg
-);
-void YSetEventToBeDisconnect(YEvent *event);
-int YSendData(int s, char *buf, int len);
-
-
-
 /*
  *	Prints YIFF network error to stream.
  */
@@ -158,7 +143,7 @@ void YNetPrintError(
         FILE *stream, YConnection *con,
         u_int16_t chunk_length,
         u_int16_t major_op_code, u_int16_t minor_op_code,
-        char *mesg
+        const char *mesg
 )
 {
 	if(stream == NULL)
@@ -203,8 +188,10 @@ void YSetEventToBeDisconnect(YEvent *event)
 	if(event == NULL)
 	    return;
 
+	/* Explicitly change event type to YDisconnect. */
 	event->type = YDisconnect;
 
+	/* Set disconnect reason code. */
 	event->disconnect.reason = 0;
 
 	return;
@@ -214,7 +201,7 @@ void YSetEventToBeDisconnect(YEvent *event)
 /*
  *	Sends binary data in buf to connection s.
  */
-int YSendData(int s, char *buf, int len)
+int YSendData(int s, const char *buf, int len)
 {
 	struct timeval timeout;
         fd_set writefds;
@@ -247,7 +234,7 @@ int YSendData(int s, char *buf, int len)
  */
 int YNetSendAudioChangePreset(
 	YConnection *con,
-	char *audio_mode_name
+	const char *audio_mode_name
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
@@ -470,7 +457,6 @@ int YNetSendDisconnect(YConnection *con, int reason)
         *(u_int16_t *)(&buf[6]) = htons((u_int16_t)0);
         *(u_int16_t *)(&buf[8]) = htons((u_int16_t)reason);
 
-
 	return(YSendData(con->fd, buf, this_seg_len));
 }
 /*
@@ -490,7 +476,6 @@ int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO)
 	    *(u_int16_t *)(&buf[8])
 	);
 
-
 	return(0);
 }
 
@@ -545,27 +530,25 @@ int YNetParseSetHost(YCNP_STD_INPUTS_PROTO)
 }
 
 /*
- *	Sends a mixer device set.
+ *	Sends a mixer channel set.
  */
 int YNetSendSetMixerChannel(
 	YConnection *con,
         int mixer_code,
-        Coefficient value1, 
-        Coefficient value2
+        Coefficient value1, Coefficient value2
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
-         *   <u16_mixercode> <u16_value1> <u16_value2>
+         *   <u16_mixercode> <u32_value1> <u32_value2>
          */
-        const YDataLength this_seg_len = 8 + 6;
+        const YDataLength this_seg_len = 8 + 10;
 
         char buf[this_seg_len];
-	u_int16_t ival1, ival2;
+	u_int32_t ival1, ival2;
 
 
-	ival1 = (Coefficient)value1 * (Coefficient)((u_int16_t)-1);
-        ival2 = (Coefficient)value2 * (Coefficient)((u_int16_t)-1);
-
+	ival1 = (u_int32_t)rint(value1 * (Coefficient)((u_int32_t)-1));
+        ival2 = (u_int32_t)rint(value2 * (Coefficient)((u_int32_t)-1));
 
         *(u_int32_t *)(&buf[0]) = htonl((u_int32_t)this_seg_len);
         *(u_int16_t *)(&buf[4]) = htons((u_int16_t)YMixerChannel);
@@ -573,13 +556,13 @@ int YNetSendSetMixerChannel(
 
         *(u_int16_t *)(&buf[8]) = htons((u_int16_t)mixer_code);
 
-        *(u_int16_t *)(&buf[10]) = htons((u_int16_t)ival1);
-        *(u_int16_t *)(&buf[12]) = htons((u_int16_t)ival2);
+        *(u_int32_t *)(&buf[10]) = htonl((u_int32_t)ival1);
+        *(u_int32_t *)(&buf[14]) = htonl((u_int32_t)ival2);
 
         return(YSendData(con->fd, buf, this_seg_len));
 }
 /*
- *      Sends a mixer device get.
+ *      Sends a mixer channel get.
  */
 int YNetSendGetMixerChannel(YConnection *con, int mixer_code)
 {
@@ -616,9 +599,9 @@ int YNetParseMixerChannel(YCNP_STD_INPUTS_PROTO)
 	else if(minor_op_code == YMixerChannelSet)
 	{
             /*   <u32_cs> <u16_majop> <u16_minop>
-             *   <u16_mixercode> <u16_value1> <u16_value2>
+             *   <u16_mixercode> <u32_value1> <u32_value2>
              */
-            const YDataLength this_seg_len = 8 + 6;
+            const YDataLength this_seg_len = 8 + 10;
 
 	    if(chunk_length < this_seg_len)
 		return(-1);
@@ -626,18 +609,18 @@ int YNetParseMixerChannel(YCNP_STD_INPUTS_PROTO)
 	    event->mixer.code = ntohs(*(u_int16_t *)(&buf[8]));
 
 	    /* Get mixer values, check availibility with defination. */
-	    if(YMixerValues >= 1)
+	    if(YMixerValues > 0)
                 event->mixer.value[0] =
-		    (Coefficient)ntohs(*(u_int16_t *)(&buf[10])) /
-		    (Coefficient)((u_int16_t)-1);
-	    if(YMixerValues >= 2)
+		    (Coefficient)ntohl(*(u_int32_t *)(&buf[10])) /
+		    (Coefficient)((u_int32_t)-1);
+	    if(YMixerValues > 1)
                 event->mixer.value[1] =
-                    (Coefficient)ntohs(*(u_int16_t *)(&buf[12])) /
-                    (Coefficient)((u_int16_t)-1);
+                    (Coefficient)ntohl(*(u_int32_t *)(&buf[14])) /
+                    (Coefficient)((u_int32_t)-1);
 	}
 
         return(0);
- }
+}
 
 /*
  *      Sends a sound play.
@@ -645,18 +628,19 @@ int YNetParseMixerChannel(YCNP_STD_INPUTS_PROTO)
 int YNetSendSoundPlay(
 	YConnection *con,
 	YID yid,
-	char *path,
+	const char *path,
 	YDataPosition pos,
 	YVolumeStruct *volume,
+	int sample_rate,
 	int repeats
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
          *   <u32_yid> <u32_pos> <u16_lvol> <u16_rvol>
-	 *   <u32_repeats>
+	 *   <u32_samplerate> <u32_repeats>
 	 *   <...path...>
          */
-        const YDataLength this_seg_len = 8 + 12 + 4 + YPathMax;
+        const YDataLength this_seg_len = 8 + 12 + 8 + YPathMax;
 
         char buf[this_seg_len];
 	YDataLength path_len, actual_seg_len;
@@ -673,11 +657,12 @@ int YNetSendSoundPlay(
         *(u_int16_t *)(&buf[16]) = htons((u_int16_t)volume->left);
         *(u_int16_t *)(&buf[18]) = htons((u_int16_t)volume->right);
 
-        *(u_int32_t *)(&buf[20]) = htonl((u_int32_t)repeats);
+        *(u_int32_t *)(&buf[20]) = htonl((u_int32_t)sample_rate);
+        *(u_int32_t *)(&buf[24]) = htonl((u_int32_t)repeats);
 
 	path_len = MIN(strlen(path), YPathMax);
 	strncpy(
-	    &buf[24],
+	    &buf[28],
 	    path,
 	    path_len
 	);
@@ -686,7 +671,7 @@ int YNetSendSoundPlay(
 	/*   Since we have a string in this seg, we need to
 	 *   calculate the actual length.
 	 */
-	actual_seg_len = 8 + 12 + 4 + path_len;
+	actual_seg_len = 8 + 12 + 8 + path_len;
 
 	/* Update segment header's chunk size! */
 	*(u_int32_t *)(&buf[0]) = htonl((u_int32_t)actual_seg_len);
@@ -756,7 +741,7 @@ int YNetParseSoundKill(YCNP_STD_INPUTS_PROTO)
  */
 int YNetSendGetSoundObjectAttributes(
 	YConnection *con,
-	char *path
+	const char *path
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
@@ -1187,12 +1172,85 @@ int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO)
 }
 
 /*
- *	Parses the buf.
+ *	Sends set sound object play values.
+ */
+int YNetSendSetSoundObjectPlayValues(
+	YConnection *con,
+	YEventSoundPlay *value
+)
+{
+        /*   <u32_cs> <u16_majop> <u16_minop>
+	 *   <u32_flags> <u32_yid>
+	 *   <u32_pos> <u32_totalrepeats>
+	 *   <u16_volleft> <u16_volright> <u32_sample_rate>
+         */
+        const YDataLength this_seg_len = 8 + 8 + 8 + 8;
+
+        char buf[this_seg_len];
+        YVolumeStruct volume;
+
+
+        volume.left = (Coefficient)value->left_volume *
+            (Coefficient)((u_int16_t)-1);
+        volume.right = (Coefficient)value->right_volume *
+            (Coefficient)((u_int16_t)-1);
+
+
+        *(u_int32_t *)(&buf[0]) = htonl((u_int32_t)this_seg_len);
+        *(u_int16_t *)(&buf[4]) = htons((u_int16_t)YSoundObjectPlayValues);
+        *(u_int16_t *)(&buf[6]) = htons((u_int16_t)YSoundObjectPlayValuesSet);
+
+	*(u_int32_t *)(&buf[8]) = htonl((u_int32_t)value->flags);
+        *(u_int32_t *)(&buf[12]) = htonl((u_int32_t)value->yid);
+
+        *(u_int32_t *)(&buf[16]) = htonl((u_int32_t)value->position);
+	*(u_int32_t *)(&buf[20]) = htonl((u_int32_t)value->total_repeats);
+
+        *(u_int16_t *)(&buf[24]) = htons((u_int16_t)volume.left);
+        *(u_int16_t *)(&buf[26]) = htons((u_int16_t)volume.right);
+        *(u_int32_t *)(&buf[28]) = htonl((u_int32_t)value->sample_rate);
+
+        return(YSendData(con->fd, buf, this_seg_len));
+}
+/*
+ *	Parses a sound object play values.
+ */
+int YNetParseSetSoundObjectPlayValues(YCNP_STD_INPUTS_PROTO)
+{
+        if(minor_op_code == YSoundObjectPlayValuesGet)
+        {
+            /* Parse sound object play values get. */
+
+            /*   <u32_cs> <u16_majop> <u16_minop>
+             */
+            const YDataLength this_seg_len = 8;
+
+            if(chunk_length < this_seg_len)
+                return(-1);
+        }
+        else if(minor_op_code == YSoundObjectPlayValuesSet)
+        {
+            /* Parse sound object play values set. */
+
+            /*   <u32_cs> <u16_majop> <u16_minop>
+             */
+            const YDataLength this_seg_len = 8;
+
+            if(chunk_length < this_seg_len)
+                return(-1);
+        }
+
+	return(0);
+}
+
+/*
+ *	Parses the buffer buf recieved from the server, this function
+ *	should be called from YNetRecv().
  */
 int YNetParse(  
 	YConnection *con,
 	YEvent *event,
-	u_int8_t *buf,
+	const u_int8_t *buf,
 	u_int32_t chunk_length, 
         u_int16_t major_op_code,
         u_int16_t minor_op_code
@@ -1264,6 +1322,10 @@ int YNetParse(
 	    status = YNetParseListAudioModes(YCNP_STD_INPUTS);
             break;
 
+	  case YSoundObjectPlayValues:
+	    status = YNetParseSetSoundObjectPlayValues(YCNP_STD_INPUTS);
+	    break;
+
 	  default:
 	    /* Unknown op code, so set event type to 0. */
 	    event->type = 0;
@@ -1279,9 +1341,17 @@ int YNetParse(
  *
  *	Returns the number of segments handled or -1 on fatal error.
  *	Fatal error implies the connection is no longer usable.
+ *
+ *	This function may close the connection and set the con->fd
+ *	to -1 if a YDisconnect or YShutdown event is recieved.
  */
 int YNetRecv(YConnection *con)
 {
+	/* Maximum events to fetch per recieve, further events are
+	 * discarded.
+	 */
+	const int max_event_fetch_loops = 10000;
+
 	int i, n, loops, status;
         struct timeval timeout;
         fd_set readfds;
@@ -1301,11 +1371,12 @@ int YNetRecv(YConnection *con)
         if(con->buf_cont < 0)
             con->buf_cont = 0;
 
+	/* Make sure that con->buf_len is greater than con->buf_cont. */
         if((con->buf_len - con->buf_cont) <= 0)
         {
-            /*   Buffer overflowed and unable to parse, so reset it
-             *   and disconnect connection since there is no way to
-             *   seek the next chunk position anymore.
+            /* Buffer overflowed and unable to parse, so reset it
+             * and disconnect connection since there is no way to
+             * seek the next chunk position anymore.
              */
             con->buf_cont = 0;
 
@@ -1348,6 +1419,11 @@ int YNetRecv(YConnection *con)
         );
         if(bytes_read == 0)
         {
+	    /* select() indicated there was data to be read but actual
+	     * recieving of data returned no data. This implies the
+	     * connection was reset by the peer.
+	     */
+
 	    /* Force disconnect. */
             if(con->fd > -1)
             {
@@ -1362,37 +1438,45 @@ int YNetRecv(YConnection *con)
             /* Handle error. */
             switch(errno)
             {
+	      /* These errors are okay, return 0 indicating no data
+	       * read but also no error.
+	       */
               case EWOULDBLOCK:
-                break;
-
               case EINTR:
+		return(0);
                 break;
 
+	      /* All other errors, force disconnect and return -1. */
               default:
                 /* Force disconnect. */
                 if(con->fd > -1)
                 {
-                  close(con->fd);
-                  con->fd = -1;
+                    close(con->fd);
+                    con->fd = -1;
                 }
 		return(-1);
                 break;
             }
-            return(0);
         }
 
-        /* Increment buffer contents. */
+        /* Increment byte count indicating contents in buffer. */
         con->buf_cont += bytes_read;
+	/* Sanitize buf_cont to the number of bytes allocated in the
+	 * buffer. This will be checked on the next loop if they
+	 * are equal and indicate an overflow if that happens.
+	 */
         if(con->buf_cont > con->buf_len)
             con->buf_cont = con->buf_len;
 
 
-	/* Begin handling each segment, handle no more than 30. */ 
+	/* Begin handling each segment, handle no more than
+	 * max_event_fetch_loops from the recieved buffer.
+	 */
 	segments_handled = 0;
-        for(loops = 0; loops < 30; loops++)
+        for(loops = 0; loops < max_event_fetch_loops; loops++)
         {
-            /*   Check if we got the first 8 bytes needed for the
-             *   chunk size, major op code, and minor op code.
+            /* Check if we did not get the first 8 bytes (needed for the
+             * chunk size, major op code, and minor op code).
              */
             if(con->buf_cont < 8)
                 break;
@@ -1401,13 +1485,18 @@ int YNetRecv(YConnection *con)
              *   <4 bytes, chunk length> <2 bytes, major op code>
              *   <2 bytes, minor op code>
              */
-            chunk_length = ntohl(*((u_int32_t *)(&con->buf[0])));
+            chunk_length = (u_int32_t)MAX(
+		(long)ntohl(*((u_int32_t *)(&con->buf[0]))), (long)0
+	    );
             major_op_code = ntohs(*((u_int16_t *)(&con->buf[4])));
             minor_op_code = ntohs(*((u_int16_t *)(&con->buf[6])));
 
-	    /* Chunk length must be 8 or greater. */
+	    /* Recieved data indicated chunk length less than 8 bytes? */
 	    if(chunk_length < 8)
 	    {
+		/* This should never happen, so consider this a fatal
+		 * error. Report it and force disconnect.
+		 */
 		YNetPrintError(
                     stderr,
                     con,
@@ -1416,29 +1505,30 @@ int YNetRecv(YConnection *con)
                     minor_op_code,
  "Recieved a segment with header specified chunk length less than 8 bytes"
                 );
-
-		/* Need to force close connection for this. */
                 if(con->fd > -1)
                 {
                     close(con->fd);
                     con->fd = -1;
                 }
-
 		return(-1);
 	    }
 
-            /*   Did we get the entire chunk? In other words does the
-             *   buffer contents value suggest that we have the entire
-             *   chunk?
+            /* Did we not get the entire chunk? In other words does the
+             * buffer contents value suggest that we do not have the
+	     * entire chunk?
              */
             if((YDataLength)chunk_length > (YDataLength)con->buf_cont)
                 break;
 
 
 	    /* Increment total number of queued events. */
+	    if(con->total_queued_events < 0)
+		con->total_queued_events = 0;
             con->total_queued_events++;
             if(con->total_queued_events > YQueuedEventsMax)
 	    {
+		/* Queued events overflowed. */
+
                 con->total_queued_events = YQueuedEventsMax;
 
                 /* Need to force close connection for this. */
@@ -1463,6 +1553,8 @@ int YNetRecv(YConnection *con)
 	    );
 	    if(con->queued_event == NULL)
 	    {
+		/* Memory allocation error. */
+
 		con->total_queued_events = 0;
 
                 /* Need to force close connection for this. */
@@ -1480,7 +1572,9 @@ int YNetRecv(YConnection *con)
 	    ]);
 
 
-	    /* Parse this segement and put data into event structure. */
+	    /* Parse recieved buffer segement and put data into event
+	     * structure event_ptr.
+	     */
             YNetParse(
                 con,
 		event_ptr,
@@ -1499,14 +1593,33 @@ int YNetRecv(YConnection *con)
                 con->buf[i] = con->buf[n];
 
             /* Decrease buffer contents. */
-            con->buf_cont = (YDataLength)con->buf_cont - (YDataLength)chunk_length;
+            con->buf_cont = (YDataLength)con->buf_cont -
+		(YDataLength)chunk_length;
             if(con->buf_cont < 0)
                 con->buf_cont = 0;
+
+
+	    /* Post event handling; check special case if the event we
+	     * just parsed is either YDisconnect or YShutdown.
+	     */
+	    if((event_ptr->type == YDisconnect) ||
+               (event_ptr->type == YShutdown)
+	    )
+	    {
+                /* Need to force close connection when we recieve either
+		 * of these events.
+		 */
+                if(con->fd > -1)
+                {
+                    close(con->fd);
+                    con->fd = -1;
+                }
+	    }
         }
 
 
-	/*   Return number of segments handled which implies events
-	 *   handled.
+	/* Return number of segments handled which implies number of
+	 * events that were handled.
 	 */
         return(segments_handled);
 }
