@@ -41,7 +41,9 @@ typedef struct {
 	/* Entire length of audio data (not size of file). */
 	YDataLength entire_length;
 
-	/* Audio library's buffer, just a pointer to the buffer. */
+	/* Audio library's buffer, this contains a *duplicate* of
+	 * the loaded DSP data from the library's structure.
+	 */
 	SoundBuffer *buffer;
 	YDataLength length;
 
@@ -62,7 +64,7 @@ typedef struct {
 
 
 
-extern int AFWOpen(char *path, AFWDataStruct *af_data);
+extern int AFWOpen(const char *path, AFWDataStruct *af_data);
 
 extern int AFWLoadSegment(
 	AFWDataStruct *af_data,
