@@ -2,7 +2,6 @@
 #define MIDI_H
 
 #include <stdio.h>
-#include <db.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -13,6 +12,8 @@
 #define MidiSuccess		0
 #define MidiBadValue		1
 #define MidiNoBuffers		2
+#define MidiNoAccess		3
+#define MidiErrorNotMidi	4
 
 
 /*
@@ -88,8 +89,8 @@ typedef struct {
 } MidiDataStruct;
 
 
-extern int MidiIsFileMidi(char *filename);
-extern int MidiRead(char *filename, MidiDataStruct *md);
+extern int MidiIsFileMidi(const char *filename);
+extern int MidiRead(const char *filename, FILE *fp, MidiDataStruct *md);
 extern void MidiDestroy(MidiDataStruct *md);
 
 
