@@ -17,16 +17,16 @@ typedef struct {
 	/* Pointer to external MIDI device module resources. */
 	void *ptr;
 
-#ifdef ALSA
-	/* Device number (specific to libkmidi. */
+#ifdef ALSA_RUN_CONFORM
+	/* ALSA MIDI device port number (needed by libkmid). */
 	int midi_device_number;
-#endif
+#endif	/* ALSA_RUN_CONFORM */
 
 } MIDIAudio;
 
 
 extern int MIDIIOWInit(MIDIAudio *midi_audio);
-extern pid_t MIDIIOWPlay(MIDIAudio *midi_audio, char *path);
+extern pid_t MIDIIOWPlay(MIDIAudio *midi_audio, const char *path);
 extern void MIDIIOWStop(MIDIAudio *midi_audio);
 extern pid_t MIDIIOWIsPlaying(MIDIAudio *midi_audio);
 extern void MIDIIOWShutdown(MIDIAudio *midi_audio);
