@@ -9,6 +9,8 @@
 
 #include <limits.h>
 
+#include "../include/Y2/Ymixercodes.h"
+
 #include "ytypes.h"
 #include "ysound.h"
 
@@ -33,12 +35,21 @@ typedef struct {
         /* Command to run midi player. */
         char *midi_play_cmd;
 
-#ifdef ALSA
-        /* Midi device number,used by the ALSA driver for multiple MIDI
-         * device support.
+#ifdef ALSA_RUN_CONFORM
+        /* ALSA MIDI device port number, used by the ALSA MIDI driver
+	 * for multiple MIDI device support. Can be -1 to indicate it
+	 * is not set.
          */
         int midi_device_number;
-#endif  /* ALSA */
+#endif  /* ALSA_RUN_CONFORM */
+
+	/* Current or last mixer channel device values for the mixer
+	 * opened on the recorder's Audio structure.
+	 */
+/* Note that we may need one for each mixer if later on there is
+ * support for multiple mixers.
+ */
+        Coefficient mixer_value[YTotalMixers][YMixerValues];
 
 } yiff_option_struct; 
 extern yiff_option_struct option;
