@@ -1,32 +1,4 @@
-/*
-                         Configuration File IO
-
-	Functions:
-
-	int RCLoadFromFile(char *filename)
-
-	void RCMixerParseMixerValueString(   
-	        char *s,
-	        Coefficient *value1,
-	        Coefficient *value2
-	)
-	void MixerFreeFData(
-	        mixer_device_fdata_struct **ptr,
-	        int total
-	)
-	mixer_device_fdata_struct **MixerRCLoadAllFromFile(
-	        char *filename,
-	        int *total
-	)
-	int MixerRCLoadFromFile(char *filename, Recorder *recorder)
-        int MixerRCSaveToFile(char *filename, Recorder *recorder)
-
-	---
-
- */
-
 #include <stdio.h>
-#include <db.h>
 #include <sys/types.h>
 #include <malloc.h>
 #include <string.h>
@@ -45,13 +17,37 @@
 #include "options.h"
 
 
+int RCLoadFromFile(const char *filename);
+
+
+/*
+ *      Structure to contain a mixer device settings read from
+ *      file.
+ */
+typedef struct {
+        char *name;
+        Coefficient val[YTotalMixers][2];
+} mixer_device_fdata_struct;
+
+static void RCMixerParseMixerValueString(
+	const char *s,
+	Coefficient *value1, Coefficient *value2
+);
+static void MixerFreeFData(
+	mixer_device_fdata_struct **ptr, int total
+);
+static mixer_device_fdata_struct **MixerRCLoadAllFromFile(
+	const char *filename, int *total
+);
+int MixerRCLoadFromFile(const char *filename, Recorder *recorder);
+int MixerRCSaveToFile(const char *filename, Recorder *recorder);
+
 
 #define MIN(a,b)	(((a) < (b)) ? (a) : (b))
 #define MAX(a,b)	(((a) > (b)) ? (a) : (b))
 
 
-
-extern int antishift(int in);	/* In main.c */
+extern int YAntiShift(int in);	/* In main.c */
 
 
 /*
@@ -60,7 +56,7 @@ extern int antishift(int in);	/* In main.c */
  *	Any current Audio modes and sound paths will be deleted
  *	and then loaded with the new ones from the file.
  */
-int RCLoadFromFile(char *filename)
+int RCLoadFromFile(const char *filename)
 {
 	int status;
         char *strptr, *strptr2;
@@ -76,7 +72,7 @@ int RCLoadFromFile(char *filename)
 	SoundPath *sp_ptr;
 
 
-	/* Check if filename exists. */
+	/* Check if file exists. */
 	if(filename == NULL)
 	    return(-1);
 	if(stat(filename, &stat_buf))
@@ -85,8 +81,8 @@ int RCLoadFromFile(char *filename)
 	    return(-1);
 	}
 
-        /* Open filename. */
-        fp = fopen(filename, "r");
+        /* Open YIFF Sound Server configuration file. */
+        fp = FOpen(filename, "rb");
         if(fp == NULL)
         {
             fprintf(stderr, "%s: Cannot open.\n", filename);
@@ -94,39 +90,51 @@ int RCLoadFromFile(char *filename)
         }
 
 
-	/* ************************************************************* */
-	/* Free any allocated resources as needed. */
+	/* Free allocated resources that would be reloaded with the
+	 * contents of the configuration file to be loaded.
+	 */
 
+	/* Y Audio Modes. */	
 	YModeDeleteAll();
+
+	/* Y Sound Paths. */
 	SoundPathDeleteAll();
 
-	/* Do not delete connections and yhosts. */
 
+/* Reads the next line, fetches parm and val. Will break the loop
+ * if end of file is reached or continue as needed. Make sure variable
+ * is initially reset to NULL before start of reading file. Uses
+ * variables; strptr, strptr2, fp, parm, val, and lines_read.
+ */
+#define DO_READ_LINE    \
+{ \
+ /* Free previous line and allocate/read next line. */ \
+ free(strptr); \
+ strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR); \
+ if(strptr == NULL) \
+  break; \
+ lines_read++; \
+ \
+ /* Fetch parameter. */ \
+ strptr2 = StringCfgParseParm(strptr); \
+ if(strptr2 == NULL) \
+  continue; \
+ strncpy(parm, strptr2, CFG_PARAMETER_MAX); \
+ parm[CFG_PARAMETER_MAX - 1] = '\0'; \
+ \
+ /* Fetch value. */ \
+ strptr2 = StringCfgParseValue(strptr); \
+ if(strptr2 == NULL) \
+  strptr2 = "0";  /* Set it to "0" if NULL. */ \
+ strncpy(val, strptr2, CFG_VALUE_MAX); \
+ val[CFG_VALUE_MAX - 1] = '\0'; \
+}
 
-	/* ******************************************************** */
-
+	/* Begin reading file. */
 	strptr = NULL;
-
         while(1)
         {
-	    /* Free previous line and allocate/read next line. */
-            free(strptr); strptr = NULL;
-            strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
-            if(strptr == NULL) break;
-            lines_read++;
-
-            /* Fetch parameter. */
-            strptr2 = StringCfgParseParm(strptr);
-            if(strptr2 == NULL) continue;
-            strncpy(parm, strptr2, CFG_PARAMETER_MAX);
-            parm[CFG_PARAMETER_MAX - 1] = '\0';
-
-            /* Fetch value. */
-            strptr2 = StringCfgParseValue(strptr);
-            if(strptr2 == NULL) strptr2 = "0";  /* Set it to "0" if NULL. */
-            strncpy(val, strptr2, CFG_VALUE_MAX);
-            val[CFG_VALUE_MAX - 1] = '\0';
-
+	    DO_READ_LINE
 
 	    /* VersionMajor */
             if(!strcasecmp(parm, "VersionMajor"))
@@ -139,19 +147,21 @@ int RCLoadFromFile(char *filename)
 
             }
 
-	    /* Port */
+	    /* Listening port number. */
             else if(!strcasecmp(parm, "Port"))
             {
 		option.port = atoi(val);
 	    }
-	    /* RefreshInterval */
+	    /* Refresh interval. */
             else if(!strcasecmp(parm, "RefreshInterval"))
             {
 		option.refresh_int.ms = atol(val) / 1000;
                 option.refresh_int.us = atol(val) % 1000;
             }
-	    /* Device */
-	    else if(!strcasecmp(parm, "Device"))
+	    /* DSP device. */
+	    else if(!strcasecmp(parm, "DSPDevice") ||
+                    !strcasecmp(parm, "Device")
+	    )
 	    {
 		strncpy(
 		    fname.device,
@@ -168,34 +178,52 @@ int RCLoadFromFile(char *filename)
                     );
                 }
 	    }
-            /* Mixer */
-            else if(!strcasecmp(parm, "Mixer"))
+            /* Mixer device. */
+            else if(!strcasecmp(parm, "MixerDevice") ||
+                    !strcasecmp(parm, "Mixer")
+	    )
             {
                 strncpy(
                     fname.mixer,
                     val,
                     PATH_MAX + NAME_MAX
                 );
-                if(stat(fname.mixer, &stat_buf))
-                {
-                    fprintf(stderr,
+		if(fname.mixer[0] != '\0')
+		{
+                    if(stat(fname.mixer, &stat_buf))
+                        fprintf(stderr,
    "%s: Line %i: Warning: %s: No such device.\n",
                             filename,
                             lines_read,
                             fname.mixer
-                    );
+                        );
                 }
             }
-	    /* MIDIPlayCommand */
+	    /* MIDI player command. */
             else if(!strcasecmp(parm, "MIDIPlayCommand"))
 	    {
 		free(option.midi_play_cmd);
-		option.midi_play_cmd = StringCopyAlloc(val);
+		option.midi_play_cmd = strdup(val);
+	    }
+	    /* ALSA MIDI device port. */
+	    else if(!strcasecmp(parm, "MIDIDevicePort"))
+            {
+#ifdef ALSA_RUN_CONFORM
+		/* If -1 then implies it is not set. */
+		if(option.midi_device_number < 0)
+		{
+		    /* Was not previously set, so set it with the
+		     * specified value from configuration file.
+		     */
+		    option.midi_device_number = atoi(val);
+		}
+#endif	/* ALSA_RUN_CONFORM */
 	    }
 
             /* **************************************************** */
-	    /* BeginAudioMode */
-            else if(!strcasecmp(parm, "BeginAudioMode") ||
+	    /* BeginYAudioMode */
+            else if(!strcasecmp(parm, "BeginYAudioMode") ||
+                    !strcasecmp(parm, "BeginAudioMode") ||
 		    !strcasecmp(parm, "BeginYMode") ||
                     !strcasecmp(parm, "BeginMode")
 	    )
@@ -207,24 +235,7 @@ int RCLoadFromFile(char *filename)
 
                 while(1)
                 {
-                    /* Free previous line and allocate/read next line. */
-                    free(strptr); strptr = NULL;
-                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
-                    if(strptr == NULL) break;
-                    lines_read++;
-                    
-                    /* Fetch parameter. */
-                    strptr2 = StringCfgParseParm(strptr);
-                    if(strptr2 == NULL) continue;
-                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
-                    parm[CFG_PARAMETER_MAX - 1] = '\0';
-                        
-                    /* Fetch value. */
-                    strptr2 = StringCfgParseValue(strptr);  
-                    if(strptr2 == NULL) strptr2 = "0";
-                    strncpy(val, strptr2, CFG_VALUE_MAX);
-                    val[CFG_VALUE_MAX - 1] = '\0';
-
+		    DO_READ_LINE
 
                     /* Name */
                     if(!strcasecmp(parm, "Name"))
@@ -286,7 +297,7 @@ int RCLoadFromFile(char *filename)
 			/*   Note: On file, it's bytes, in memory it's
                          *   shifts.
                          */
-                        ymode_ptr->fragment_size = antishift(atoi(val) - 1);
+                        ymode_ptr->fragment_size = YAntiShift(atoi(val) - 1);
                     }
 #endif  /* OSS_BUFFRAG */
                     /* FlipStereo */
@@ -303,8 +314,9 @@ int RCLoadFromFile(char *filename)
                             ymode_ptr->direction = AUDIO_DIRECTION_PLAY;
                     }
 
-                    /* EndMode */
-                    else if(!strcasecmp(parm, "EndAudioMode") ||
+                    /* EndYAudioMode */
+                    else if(!strcasecmp(parm, "EndYAudioMode") ||
+                            !strcasecmp(parm, "EndAudioMode") ||
 			    !strcasecmp(parm, "EndYMode") ||
                             !strcasecmp(parm, "EndMode")
 		    )
@@ -328,8 +340,10 @@ int RCLoadFromFile(char *filename)
             }
 
             /* **************************************************** */
-            /* BeginSoundPath */
-            else if(!strcasecmp(parm, "BeginSoundPath"))
+            /* BeginYSoundPath */
+            else if(!strcasecmp(parm, "BeginYSoundPath") ||
+                    !strcasecmp(parm, "BeginSoundPath")
+	    )
             {
                 status = SoundPathAllocate();
                 sp_ptr = SoundPathGetPtr(status);
@@ -338,24 +352,7 @@ int RCLoadFromFile(char *filename)
 
                 while(1)
                 {
-                    /* Free previous line and allocate/read next line. */
-                    free(strptr); strptr = NULL;
-                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
-                    if(strptr == NULL) break;
-                    lines_read++;
-
-                    /* Fetch parameter. */
-                    strptr2 = StringCfgParseParm(strptr);
-                    if(strptr2 == NULL) continue;
-                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
-                    parm[CFG_PARAMETER_MAX - 1] = '\0';
-
-                    /* Fetch value. */
-                    strptr2 = StringCfgParseValue(strptr);
-                    if(strptr2 == NULL) strptr2 = "0";
-                    strncpy(val, strptr2, CFG_VALUE_MAX);
-                    val[CFG_VALUE_MAX - 1] = '\0';  
-
+		    DO_READ_LINE
 
                     /* Path */   
                     if(!strcasecmp(parm, "Path"))
@@ -363,8 +360,10 @@ int RCLoadFromFile(char *filename)
                         free(sp_ptr->path);
                         sp_ptr->path = StringCopyAlloc(val);
                     }
-                    /* EndSoundPath */
-                    else if(!strcasecmp(parm, "EndSoundPath"))
+                    /* EndYSoundPath */
+                    else if(!strcasecmp(parm, "EndYSoundPath") ||
+                            !strcasecmp(parm, "EndSoundPath")
+		    )
                     {
                         break;
                     }
@@ -393,74 +392,72 @@ int RCLoadFromFile(char *filename)
             }
 	}
 
+#undef DO_READ_LINE
 
 	/* Close the file. */
-	fclose(fp);
+	FClose(fp);
 
 
 	return(0);
 }
 
-
-
-/*
- *      Structure to contain a mixer device settings read from
- *	file.
- */
-typedef struct {
-            
-        char *name;
-        Coefficient val[YTotalMixers][2];
-                
-} mixer_device_fdata_struct;
-
 /*
  *	Parses two values seperated by a white space from string
  *	s and puts them into the Coefficient values.
  */
-void RCMixerParseMixerValueString(
-	char *s,
-	Coefficient *value1,
-	Coefficient *value2
+static void RCMixerParseMixerValueString(
+	const char *s,
+	Coefficient *value1, Coefficient *value2
 )
 {
+	const char *cstrptr;
 	char *strptr;
 	const int len = 256;
 	char ls[len];
 
 
 	/* Get value 1. */
-	strptr = s;
-	if(strptr != NULL)
+	cstrptr = s;
+	if(cstrptr != NULL)
         {
-            while(*strptr == ' ')
-                strptr++;
+            while(ISBLANK(*cstrptr))
+                cstrptr++;
 
-            strncpy(ls, strptr, len);
+            strncpy(ls, cstrptr, len);
 	    ls[len - 1] = '\0';
 
 	    strptr = strchr(ls, ' ');
 	    if(strptr != NULL)
-		*strptr = '\0';
-	    *value1 = atof(ls);
+		(*strptr) = '\0';
+            strptr = strchr(ls, '\t');
+            if(strptr != NULL)
+                (*strptr) = '\0';
+
+	    (*value1) = atof(ls);
 	}
 
 	/* Get value 2. */
-	strptr = strchr(s, ' ');
-	if(strptr != NULL)
-	{
-            while(*strptr == ' ')
-                strptr++;
+	cstrptr = strchr(s, ' ');
+	if(cstrptr == NULL)
+	    cstrptr = strchr(s, '\t');
 
-	    strncpy(ls, strptr, len);
+	if(cstrptr != NULL)
+	{
+            while(ISBLANK(*cstrptr))
+                cstrptr++;
+
+	    strncpy(ls, cstrptr, len);
             ls[len - 1] = '\0';
 
             strptr = strchr(ls, ' ');
             if(strptr != NULL) 
-                *strptr = '\0';
-            *value2 = atof(ls);
-	}
+                (*strptr) = '\0';
+            strptr = strchr(ls, '\t');
+            if(strptr != NULL)
+                (*strptr) = '\0';
 
+            (*value2) = atof(ls);
+	}
 
 	return;
 }
@@ -468,20 +465,19 @@ void RCMixerParseMixerValueString(
 /*
  *	Deallocates mixer device file data list.
  */
-void MixerFreeFData(
-	mixer_device_fdata_struct **ptr,
-	int total
-)
+static void MixerFreeFData(mixer_device_fdata_struct **ptr, int total)
 {
 	int i;
+	mixer_device_fdata_struct *mdfd_ptr;
 
 	for(i = 0; i < total; i++)
 	{
-	    if(ptr[i] == NULL)
+	    mdfd_ptr = ptr[i];
+	    if(mdfd_ptr == NULL)
 		continue;
 
-	    free(ptr[i]->name);
-	    free(ptr[i]);
+	    free(mdfd_ptr->name);
+	    free(mdfd_ptr);
 	}
 	free(ptr);
 
@@ -489,17 +485,17 @@ void MixerFreeFData(
 }
 
 /*
- *	Loads mixer devices and their values from the file,
- *	allocates them in the returned pointer array.
+ *	Loads all mixer device channel values from the specified
+ *	file and returns an allocated array of listings for each
+ *	device.
  *
- *	This array needs to be free'ed by teh calling function.
+ *	This array needs to be free'ed by the calling function.
  */
-mixer_device_fdata_struct **MixerRCLoadAllFromFile(
-	char *filename,
-	int *total
+static mixer_device_fdata_struct **MixerRCLoadAllFromFile(
+	const char *filename, int *total
 )
 {
-        int i, n, status;
+        int i, n;
         char *strptr, *strptr2;
 
         FILE *fp;
@@ -508,64 +504,76 @@ mixer_device_fdata_struct **MixerRCLoadAllFromFile(
         char parm[CFG_PARAMETER_MAX];
         char val[CFG_VALUE_MAX];
    
-        Coefficient value1, value2;
         char *mixer_name[] = YMixerConicalNames;
 
 	mixer_device_fdata_struct **ptr = NULL;
 	mixer_device_fdata_struct *mdfd_ptr;
 
 
-        /* Error checks. */
         if((filename == NULL) ||
            (total == NULL)
         )
             return(NULL);
+
         if(stat(filename, &stat_buf))
             return(NULL);
 
 	/* Reset total. */
-	*total = 0;
+	(*total) = 0;
 
-        /* Open file for reading. */
-        fp = fopen(filename, "r");
+        /* Open mixer device file for reading. */
+        fp = FOpen(filename, "rb");
         if(fp == NULL)
             return(NULL);
 
+/* Reads the next line, fetches parm and val. Will break the loop
+ * if end of file is reached or continue as needed. Make sure variable
+ * is initially reset to NULL before start of reading file. Uses
+ * variables; strptr, strptr2, fp, parm, val, and lines_read.
+ */
+#define DO_READ_LINE	\
+{ \
+ /* Free previous line and allocate/read next line. */ \
+ free(strptr); \
+ strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR); \
+ if(strptr == NULL) \
+  break; \
+ lines_read++; \
+ \
+ /* Fetch parameter. */ \
+ strptr2 = StringCfgParseParm(strptr); \
+ if(strptr2 == NULL) \
+  continue; \
+ strncpy(parm, strptr2, CFG_PARAMETER_MAX); \
+ parm[CFG_PARAMETER_MAX - 1] = '\0'; \
+ \
+ /* Fetch value. */ \
+ strptr2 = StringCfgParseValue(strptr); \
+ if(strptr2 == NULL) \
+  strptr2 = "0";  /* Set it to "0" if NULL. */ \
+ strncpy(val, strptr2, CFG_VALUE_MAX); \
+ val[CFG_VALUE_MAX - 1] = '\0'; \
+}
+
         /* Begin reading file. */
         strptr = NULL;
         while(1)
         {
-            /* Free previous line and allocate/read next line. */
-            free(strptr); strptr = NULL;
-            strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
-            if(strptr == NULL) break;
-            lines_read++;
-
-            /* Fetch parameter. */
-            strptr2 = StringCfgParseParm(strptr);
-            if(strptr2 == NULL) continue;
-            strncpy(parm, strptr2, CFG_PARAMETER_MAX);
-            parm[CFG_PARAMETER_MAX - 1] = '\0';
-
-            /* Fetch value. */  
-            strptr2 = StringCfgParseValue(strptr);
-            if(strptr2 == NULL) strptr2 = "0";  /* Set it to "0" if NULL. */
-            strncpy(val, strptr2, CFG_VALUE_MAX);
-            val[CFG_VALUE_MAX - 1] = '\0';
+	    DO_READ_LINE
 
             /* BeginMixer */
             if(!strcasecmp(parm, "BeginMixer"))
             {
 		/* Allocate a new mixer device structure. */
-		i = *total;
-		*total = i + 1;
+		i = (*total);
+		(*total) = i + 1;
 		ptr = (mixer_device_fdata_struct **)realloc(
 		    ptr,
-		    *total * sizeof(mixer_device_fdata_struct *)
+		    (*total) * sizeof(mixer_device_fdata_struct *)
 		);
 		if(ptr == NULL)
 		{
-		    *total = 0;
+		    (*total) = 0;
 		    continue;
 		}
 
@@ -575,7 +583,7 @@ mixer_device_fdata_struct **MixerRCLoadAllFromFile(
 		);
 		if(ptr[i] == NULL)
 		{
-		    *total = i;
+		    (*total) = i;
 		    continue;
 		}
 		mdfd_ptr = ptr[i];
@@ -586,24 +594,7 @@ mixer_device_fdata_struct **MixerRCLoadAllFromFile(
 
                 while(1)
 		{
-                    /* Free previous line and allocate/read next line. */
-                    free(strptr); strptr = NULL;
-                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
-                    if(strptr == NULL) break;
-                    lines_read++;
-        
-                    /* Fetch parameter. */
-                    strptr2 = StringCfgParseParm(strptr);
-                    if(strptr2 == NULL) continue;
-                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
-                    parm[CFG_PARAMETER_MAX - 1] = '\0';
-            
-                    /* Fetch value. */
-                    strptr2 = StringCfgParseValue(strptr);
-                    if(strptr2 == NULL) strptr2 = "0";
-                    strncpy(val, strptr2, CFG_VALUE_MAX);
-                    val[CFG_VALUE_MAX - 1] = '\0';
-
+		    DO_READ_LINE
 
 		    /* Go through mixer device channel names. */  
                     for(n = 0; n < YTotalMixers; n++)
@@ -627,17 +618,20 @@ mixer_device_fdata_struct **MixerRCLoadAllFromFile(
             /* Unknown parameter. */
             else
             {
-                fprintf(stderr, "%s: Line %i: Unknown parameter: %s\n",
+                fprintf(
+		    stderr,
+		    "%s: Line %i: Unknown parameter: %s\n",
                     filename,
                     lines_read,
-                    parm 
-                );   
-            }       
-        }           
+                    parm
+                );
+            }
+        }
 
-        /* Close the file. */
-        fclose(fp);
+#undef DO_READ_LINE
 
+        /* Close mixer device data file. */
+        FClose(fp);
 
         return(ptr);
 }
@@ -649,34 +643,41 @@ mixer_device_fdata_struct **MixerRCLoadAllFromFile(
  *
  *	The recorder's mixer must already be opened for this
  *	function to have any affect.
+ *
+ *	If global fname.mixer is an empty string then no mixer data
+ *	will be loaded.
  */
-int MixerRCLoadFromFile(char *filename, Recorder *recorder)
+int MixerRCLoadFromFile(const char *filename, Recorder *recorder)
 {
-	int i, n, total;
+	int i, mixer_index, total;
+	int set_our_mixer = 0;
 	mixer_device_fdata_struct **ptr, *mdfd_ptr;
-	char *this_mixer_dev_name;
-        char *mixer_name[] = YMixerConicalNames; 
+	const char *this_mixer_dev_name;
 
 
-	/* Error checks. */
-        if((filename == NULL) ||
-           (recorder == NULL)
-	)
+        if((filename == NULL) || (recorder == NULL))
             return(-1);
 
-        /* Get pointer to our mixer device's name. */
+        /* Get pointer to our mixer device's name, this will be
+	 * used to check through the listing of mixers to see which
+	 * one we want to load values for.
+	 */
         this_mixer_dev_name = fname.mixer;
+	if(this_mixer_dev_name[0] == '\0')
+	{
+	    /* No mixer device to load. */
+	    return(0);
+	}
 
+        /* Load all mixer devices data from file. */
+	ptr = MixerRCLoadAllFromFile(filename, &total);
+        if(ptr == NULL)
+            total = 0;
 
-	/* Read all mixer device names from file. */
-	ptr = MixerRCLoadAllFromFile(
-	    filename,
-	    &total
-	);
-	if((ptr == NULL) || (total <= 0))
+	/* No mixer devices loaded? */
+	if(total <= 0)
 	    return(-1);
 
-
 	/* Go through mixer device listing. */
 	for(i = 0; i < total; i++)
 	{
@@ -687,31 +688,32 @@ int MixerRCLoadFromFile(char *filename, Recorder *recorder)
 	    if(mdfd_ptr->name == NULL)
 		continue;
 
-
-	    /* Is this device listing the same as the one
-	     * we're looking for?
-	     */
-	    if(strcasecmp(mdfd_ptr->name, this_mixer_dev_name))
-		continue;
-
-	    /* Go through mixer device channel settings. */
-	    for(n = 0; n < YTotalMixers; n++)
+	    /* Match mixer device name. */
+	    if(!strcasecmp(mdfd_ptr->name, this_mixer_dev_name))
 	    {
-                /* Set mixer values for our mixer. */
-		YMixerSet(
-		    recorder,
-		    n + YMixerCodeBaseOffset,
-		    mdfd_ptr->val[n][0],
-		    mdfd_ptr->val[n][1]
-		);
+		/* This is the mixer device we're looking for, now go
+		 * through each mixer device channel setting.
+		 */
+		for(mixer_index = 0; mixer_index < YTotalMixers; mixer_index++)
+		{
+		    /* Set mixer values for our mixer. */
+		    YMixerSet(
+			recorder, mixer_index + YMixerCodeBaseOffset,
+			mdfd_ptr->val[mixer_index]
+		    );
+		}
+
+		set_our_mixer = 1;
 	    }
 	}
 
-	/* Deallocate mixer data listing. */
+	/* Deallocate the loaded mixer data listing. */
 	MixerFreeFData(ptr, total);
 
-
-	return(0);
+	if(set_our_mixer)
+	    return(0);
+	else
+	    return(-1);
 }
 
 /*
@@ -719,42 +721,48 @@ int MixerRCLoadFromFile(char *filename, Recorder *recorder)
  *	recorder to file.
  *
  *	Other mixer device channel values in the file will also
- *	be read and rewritten to file (but their values unmodified).
+ *	be read and rewritten to file (their values will be
+ *	preserved).
+ *
+ *	If the fname.mixer is an empty string then no operation
+ *	will be performed.
  *
  *	The recorder's mixer must already be opened for this
  *	function to have any affect.
  */
-int MixerRCSaveToFile(char *filename, Recorder *recorder)
+int MixerRCSaveToFile(const char *filename, Recorder *recorder)
 {
-        int i, n, total;
+        int i, mixer_index, total;
+	int got_our_mixer = 0;
 	FILE *fp;
         mixer_device_fdata_struct **ptr, *mdfd_ptr;
-        char *this_mixer_dev_name;
+	const char *this_mixer_dev_name;
         char *mixer_name[] = YMixerConicalNames;
 
 
-        /* Error checks. */
         if((filename == NULL) ||
            (recorder == NULL)
         )
             return(-1);
 
-
-        /* Set this mixer name pointer. */
+        /* Get pointer to our mixer device's name, this will be
+         * used to check through the listing of mixers to see which
+         * one we want to save values for.
+         */
         this_mixer_dev_name = fname.mixer;
- 
+	if(this_mixer_dev_name[0] == '\0')
+	{
+	    /* Mixer device name is empty string, nothing to save. */
+	    return(0);
+	}
 
-        /* Read all mixer device names from file. */
-        ptr = MixerRCLoadAllFromFile(
-            filename,
-            &total
-        );
-        if((ptr == NULL) || (total <= 0))   
-            return(-1);
+        /* Load all mixer devices data from file. */
+        ptr = MixerRCLoadAllFromFile(filename, &total);
+        if(ptr == NULL)
+	    total = 0;
 
-
-        /* Go through mixer device listing, search for our
-	 * mixer device by name. If it is found, then update
+        /* Go through mixer device listing, search for our mixer
+	 * device by matching name. If it is found, then update
 	 * its values with the current mixer device channel values.
 	 */
         for(i = 0; i < total; i++)
@@ -766,34 +774,84 @@ int MixerRCSaveToFile(char *filename, Recorder *recorder)
             if(mdfd_ptr->name == NULL)
                 continue;
 
-
-            /* Is this device listing the same as the one
-             * we're looking for?  
-             */
-            if(strcasecmp(mdfd_ptr->name, this_mixer_dev_name))
-                continue;
-
-            /* Go through mixer device channel settings. */
-            for(n = 0; n < YTotalMixers; n++)
-            {
-		/* Get current mixer values for our mixer. */
-                YMixerGet(
-                    recorder,
-                    n + YMixerCodeBaseOffset,
-                    &mdfd_ptr->val[n][0],
-                    &mdfd_ptr->val[n][1]
-                );
+            /* Match mixer device name. */
+            if(!strcasecmp(mdfd_ptr->name, this_mixer_dev_name))
+	    {
+                /* This is the mixer device we're looking for, now go
+                 * through each mixer device channel setting.
+                 */
+		for(mixer_index = 0; mixer_index < YTotalMixers; mixer_index++)
+		{
+		    /* Get current mixer values for our mixer, store
+		     * the fetched values into the mixer device file
+		     * data structure which we will save later.
+		     */
+		    YMixerGet(
+			recorder, mixer_index + YMixerCodeBaseOffset,
+			mdfd_ptr->val[mixer_index]
+		    );
+		}
             }
-        }    
+
+	    /* Mark that we have set values for our mixer device
+	     * in the list.
+	     */
+	    got_our_mixer = 1;
+        }
+	/* If our mixer device was not in the loaded mixer device listing
+	 * then append it to the list.
+         */
+	if(!got_our_mixer)
+	{
+	    /* Sanitize total. */
+	    if(total < 0)
+		total = 0;
+
+	    /* Allocate one more pointer. */
+	    i = total;
+	    total++;
+            ptr = (mixer_device_fdata_struct **)realloc(
+		ptr,
+		total * sizeof(mixer_device_fdata_struct *)
+	    );
+	    if(ptr == NULL)
+	    {
+		total = 0;
+	    }
+	    else
+	    {
+		/* Allocate our mixer device structure in the list. */
+		ptr[i] = (mixer_device_fdata_struct *)calloc(
+		    1,
+		    sizeof(mixer_device_fdata_struct)
+		);
+		mdfd_ptr = ptr[i];
+		if(mdfd_ptr != NULL)
+		{
+		    mdfd_ptr->name = strdup(this_mixer_dev_name);
+
+                    /* Go through mixer device channel settings. */
+                    for(mixer_index = 0; mixer_index < YTotalMixers; mixer_index++)
+                    {
+                        /* Get current mixer values for our mixer, store
+                         * the fetched values into the mixer device file
+                         * data structure which we will save later.
+                         */   
+                        YMixerGet(
+                            recorder, mixer_index + YMixerCodeBaseOffset,
+                            mdfd_ptr->val[mixer_index]
+                        );
+                    }
+		}
+	    }
+	}
 
 	/* Do not deallocate mixer device listing just yet, we need
 	 * to write them back to file below.
 	 */
 
-	/* ************************************************************ */
-
-        /* Open file for writing */
-        fp = fopen(filename, "w");
+        /* Open mixer device file for writing */
+        fp = FOpen(filename, "wb");
         if(fp != NULL)
 	{
 	    /* Write commented heading. */
@@ -819,12 +877,14 @@ int MixerRCSaveToFile(char *filename, Recorder *recorder)
 		);
 
 		/* Write each mixer channel values. */
-		for(n = 0; n < YTotalMixers; n++)
+		for(mixer_index = 0; mixer_index < YTotalMixers; mixer_index++)
 		{
-		    fprintf(fp, "    %s = %.4lf %.4lf\n",
-			mixer_name[n],
-			mdfd_ptr->val[n][0],
-			mdfd_ptr->val[n][1]
+		    fprintf(fp, "    %s = %f %f\n",
+			mixer_name[mixer_index],
+			(YMixerValues > 0) ?
+			    mdfd_ptr->val[mixer_index][0] : 0.0,
+			(YMixerValues > 1) ?
+			    mdfd_ptr->val[mixer_index][1] : 0.0
 		    );
 		}
 
@@ -832,13 +892,13 @@ int MixerRCSaveToFile(char *filename, Recorder *recorder)
 	    }
 
 	    /* Close the file. */
-	    fclose(fp);
+	    FClose(fp);
+	    fp = NULL;
 	}
 
-        /* Deallocate mixer data listing. */
+        /* Deallocate loaded mixer device file data listing. */
         MixerFreeFData(ptr, total);
 
-
 	return(0);
 }
 
