@@ -22,7 +22,7 @@ extern int total_soundpaths;
 extern int SoundPathIsAllocated(int n);
 extern SoundPath *SoundPathGetPtr(int n);
 
-extern char *SoundPathCompletePath(char *filename);
+extern char *SoundPathCompletePath(const char *filename);
 
 extern int SoundPathAllocate();
 extern void SoundPathDelete(int n);
