@@ -1,28 +1,4 @@
-/*
-                        VOC File Format Library
-	Functions:
-
-	FILE *VocOpenFile(char *filename, char *mode, off_t *size_rtn)
-	int VocIsFileVoc(char *filename)
-	unsigned int VocGetEnvSampleRate(voc_data_struct *vd)
-	void VocDestroyData(voc_data_struct *vd)
-	int VocAddDataBlock(voc_data_struct *vd, unsigned char type)
-	int VocReadHeader(char *filename, voc_data_struct *vd)
-	int VocReadPartialData(  
-	        voc_data_struct *vd,
-	        off_t offset,  
-	        off_t max_chunk_size,
-	        int read_opt
-	)
-
-
-
-	---
-
- */
-
 #include <stdio.h>
-#include <malloc.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -32,8 +8,30 @@
 #include "voc.h"
 
 
+static FILE *VocOpenFile(
+	const char *filename, const char *mode, off_t *size_rtn
+);
+int VocIsFileVoc(const char *filename);
+unsigned int VocGetEnvSampleRate(voc_data_struct *vd);
 
-FILE *VocOpenFile(char *filename, char *mode, off_t *size_rtn)
+void VocDestroyData(voc_data_struct *vd);
+int VocAddDataBlock(voc_data_struct *vd, int type);
+
+int VocReadHeader(const char *filename, FILE *fp, voc_data_struct *vd);
+int VocReadPartialData(
+	voc_data_struct *vd,
+	off_t offset,
+	off_t max_chunk_size,
+	int read_opt
+);
+
+
+/*
+ *	Procedure to open a voc file and return its entire file size.
+ */
+static FILE *VocOpenFile(
+	const char *filename, const char *mode, off_t *size_rtn
+)
 {
 
         FILE *fp;
@@ -69,15 +67,15 @@ FILE *VocOpenFile(char *filename, char *mode, off_t *size_rtn)
             return(NULL);
         }
 
-
         return(fp);
 }
 
-
-
-int VocIsFileVoc(char *filename)
+/*
+ *	Checks if the specified file is a VOC file.
+ */
+int VocIsFileVoc(const char *filename)
 {
-	int x;
+	int i;
 	char magicname[0x14];
 	FILE *fp;
 	off_t filesize;
@@ -90,47 +88,52 @@ int VocIsFileVoc(char *filename)
         if(filesize == 0)
             return(VocErrorNoAccess);
 
-
 	/* Bytes 0x00 to 0x13: Get magic name. */
-	for(x = 0; x < 0x13; x++)
-	{
-	    magicname[x] = (char)fgetc(fp);
-	}
+	for(i = 0; i < 0x13; i++)
+	    magicname[i] = (char)fgetc(fp);
 
 	/* Close file. */
-	fclose(fp); fp = NULL;
+	fclose(fp);
 
 	/* Check magic name. */
 	magicname[0x13] = '\0';
-	if(strcmp(magicname, VocMagicName) == 0)
-	    return(VocSuccess);
-	else
+	if(strcmp(magicname, VocMagicName))
 	    return(VocErrorNotVoc);
+	else
+	    return(VocSuccess);
 }
 
-
+/*
+ *	Returns the sample rate of that of the first data block
+ *	of type VocBlockTypeSoundData that is found on the blocks
+ *	in the vd structure.
+ *
+ *	If vd is invalid or no blocks of type VocBlockTypeSoundData exist
+ *	then the value VocDefaultSampleRate will be returned.
+ */
 unsigned int VocGetEnvSampleRate(voc_data_struct *vd)
 {
-	int x;
-	unsigned int sample_rate;
+	int i;
+	unsigned int sample_rate = VocDefaultSampleRate;
+	voc_block_struct **ptr, *block_ptr;
 
 
-	/* Set default. */
-	sample_rate = VocDefaultSampleRate;
-
 	if(vd == NULL)
 	    return(sample_rate);
 
-
-	for(x = 0; x < vd->total_datablocks; x++)
+	/* Go through each data block. */
+	for(i = 0, ptr = vd->datablock;
+            i < vd->total_datablocks;
+            i++, ptr++)
 	{
-	    if(vd->datablock[x] == NULL)
+	    block_ptr = *ptr;
+	    if(block_ptr == NULL)
 		continue;
 
 	    /* Check if this type is a VocBlockTypeSoundData. */
-	    if(vd->datablock[x]->type == VocBlockTypeSoundData)
+	    if(block_ptr->type == VocBlockTypeSoundData)
 	    {
-		sample_rate = vd->datablock[x]->sample_rate;
+		sample_rate = block_ptr->sample_rate;
 		break;
 	    }
 	}
@@ -139,48 +142,54 @@ unsigned int VocGetEnvSampleRate(voc_data_struct *vd)
 	if(sample_rate == 0)
 	    sample_rate = VocDefaultSampleRate;
 
-
 	return(sample_rate);
 }
 
-
-
+/*
+ *	Deallocates all resources and closes any opened files specified
+ *	in the given vd structure and resets its values.
+ */
 void VocDestroyData(voc_data_struct *vd)
 {
 	int i;
+	voc_block_struct **ptr, *block_ptr;
 
 
 	if(vd == NULL)
 	    return;
 
-
-	/* Free datablocks first. */
-	if((vd->datablock != NULL) &&
-           (vd->total_datablocks > 0)
+	/* Data blocks. */
+	for(i = 0, ptr = vd->datablock;
+            i < vd->total_datablocks;
+            i++, ptr++
 	)
 	{
-	    for(i = 0; i < vd->total_datablocks; i++)
-	    {
-		if(vd->datablock[i] == NULL)
-		    continue;
+	    block_ptr = *ptr;
+	    if(block_ptr == NULL)
+		continue;
 
-		free(vd->datablock[i]->str);
+	    free(block_ptr->str);
 
-		free(vd->datablock[i]);
-	    }
-
-	    free(vd->datablock);
-	    vd->datablock = NULL;
-
-	    vd->total_datablocks = 0;
+	    free(block_ptr);
 	}
 
+	free(vd->datablock);
+	vd->datablock = NULL;
+
+	vd->total_datablocks = 0;
+
 
 	/* Reset values. */
 	free(vd->filename);
 	vd->filename = NULL;
 	vd->filesize = 0;
 
+	if(vd->fp != NULL)
+	{
+	    fclose(vd->fp);
+	    vd->fp = NULL;
+	}
+
         free(vd->data);
 	vd->data = NULL;
         vd->data_len = 0;
@@ -195,31 +204,27 @@ void VocDestroyData(voc_data_struct *vd)
 	return;
 }
 
-
-
-int VocAddDataBlock(voc_data_struct *vd, unsigned char type)
+/*
+ *	Allocates a new data block on the vd structure by the specified
+ *	type.
+ */
+int VocAddDataBlock(voc_data_struct *vd, int type)
 {
-	int prev_total_blocks;
+	int n;
 
 
-	/* Error checks. */
 	if(vd == NULL)
 	    return(-1);
 
-	/* Sanitize total. */
 	if(vd->total_datablocks < 0)
 	    vd->total_datablocks = 0;
 
-	/* Record previous total. */
-	prev_total_blocks = vd->total_datablocks;
-
-	/* Set new total. */
+	n = vd->total_datablocks;
 	vd->total_datablocks++;
 
-
-	/* Realloc pointers. */
 	vd->datablock = (voc_block_struct **)realloc(
-	    vd->datablock, vd->total_datablocks * sizeof(voc_block_struct *)
+	    vd->datablock,
+	    vd->total_datablocks * sizeof(voc_block_struct *)
 	);
 	if(vd->datablock == NULL)
 	{
@@ -227,30 +232,37 @@ int VocAddDataBlock(voc_data_struct *vd, unsigned char type)
 	    return(-1);
 	}
 
-
-	/* Allocate new data block. */
-	vd->datablock[prev_total_blocks] = (voc_block_struct *)calloc(1,
+	vd->datablock[n] = (voc_block_struct *)calloc(
+	    1,
 	    sizeof(voc_block_struct)
 	);
-	if(vd->datablock[prev_total_blocks] == NULL)
+	if(vd->datablock[n] == NULL)
 	{
-	    vd->total_datablocks = prev_total_blocks;
+	    vd->total_datablocks = n;
 	    return(-1);
 	}
 
-
 	/* Set type. */
-	vd->datablock[prev_total_blocks]->type = type;
+	vd->datablock[n]->type = type;
 
-
-	return(prev_total_blocks);
+	return(n);
 }
 
-
-
-int VocReadHeader(char *filename, voc_data_struct *vd)
+/*
+ *      Reads the header from the stream fp and initializes the given
+ *      vd structure if fp is valid and a voc file.
+ *
+ *      The given filename is used only for referance purposes, it can be
+ *	NULL.
+ *
+ *      If VocSuccess is returned then the given fp will be transfered
+ *      to the vd structure and should not be referanced again. For all
+ *      other return values, the calling function is responsible for
+ *      closing the given fp.
+ */
+int VocReadHeader(const char *filename, FILE *fp, voc_data_struct *vd)
 {
-	int x, y, z, len;
+	int x, y, z;
 	unsigned char type;
 	off_t nextseek;
 
@@ -261,59 +273,55 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 	unsigned int size_x, size_y, size_z;
 
 	int bytes_read = 0;
-	FILE *fp;
+	struct stat stat_buf;
 	off_t filesize;
 
 
-        /* Reset values. */
-	memset(vd, 0x00, sizeof(voc_data_struct));
-
-
         /* Error checks. */
         if(vd == NULL)
-            return(VocErrorBadValue);
-        if(filename == NULL)
-            return(VocErrorBadValue);
-
-
-	/* Open file. */
-	fp = VocOpenFile(filename, "rb", &filesize);
+            return(VocErrorNoBuffers);
         if(fp == NULL)
+            return(VocErrorBadValue);
+
+        /* Rewind to beginning of voc file. */
+        rewind(fp);
+
+
+	/* Reset values. */
+        memset(vd, 0x00, sizeof(voc_data_struct));
+
+        /* Get size of file. */
+        if(fstat(fileno(fp), &stat_buf))
             return(VocErrorNoAccess);
+
+        filesize = stat_buf.st_size;
         if(filesize == 0)
             return(VocErrorNoAccess);
 
+        /* Is this is a voc file? Check 0x00 to 0x12 for magic number. */
+        for(x = 0; x < 0x13; x++)
+        {
+            magicname[x] = (char)fgetc(fp);
+            bytes_read++;
+        }
+        magicname[0x13] = '\0';
+        /* Magic name does not match? If so then this is not a voc
+	 * file.
+	 */
+        if(strcmp(magicname, VocMagicName))
+            return(VocErrorNotVoc);
+
+
+	/* Begin reading voc file header. */
 
         /* Record file name. */
-	len = strlen(filename);
-        vd->filename = (char *)malloc((len + 1) * sizeof(char));
-	if(vd->filename != NULL)
-	{
-            strncpy(vd->filename, filename, len);
-	    vd->filename[len] = '\0';
-	}
+        if(filename != NULL)
+            vd->filename = strdup(filename);
 
 	/* Record file size. */
 	vd->filesize = filesize;
 
 
-	/* *********************************************************** */
-
-	/* 0x00 to 0x12: Magic number (er name for VOC files). */
-	for(x = 0; x < 0x13; x++)
-	{
-	    magicname[x] = (char)fgetc(fp);
-	    bytes_read++;
-	}
-	magicname[0x13] = '\0';
-	/* Check to see if this is a Voc file! */
-	if(strcmp(magicname, VocMagicName) != 0)
-	{
-	    VocDestroyData(vd);
-	    return(VocErrorNotVoc);
-	}
-
-
 	/* 0x13: Read but skip abort print byte. */
 	fgetc(fp);
 	bytes_read++;
@@ -348,7 +356,8 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 	    fseek(fp, nextseek, SEEK_SET);
 
 	    /* Read type. */
-	    type = (unsigned char)fgetc(fp); bytes_read++;
+	    type = (unsigned char)fgetc(fp);
+	    bytes_read++;
 	    switch(type)
 	    {
               case VocBlockTypeTerminator:
@@ -392,7 +401,8 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 		    vd->datablock[y]->data_len = 2;
 
 		/* Get sample rate. */
-		z = fgetc(fp); bytes_read++;
+		z = fgetc(fp);
+		bytes_read++;
 		if(z >= 256)
 		    z = 255;
 		vd->datablock[y]->sample_rate = (unsigned int)(
@@ -400,8 +410,8 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 		);
 
 		/* Get compresion type. */
-		vd->datablock[y]->compression_type = (unsigned char)
-		    fgetc(fp); bytes_read++;
+		vd->datablock[y]->compression_type = (unsigned char)fgetc(fp);
+		bytes_read++;
 
 		/* Calculate bits. */
 		switch(vd->datablock[y]->compression_type)
@@ -479,7 +489,8 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 		bytes_read += 2;
 
 		/* Get sample rate. */
-                z = fgetc(fp); bytes_read++;
+                z = fgetc(fp);
+		bytes_read++;
                 if(z >= 256)
                     z = 255;
                 vd->datablock[y]->sample_rate = (unsigned int)(
@@ -631,12 +642,6 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 	    }
 	}
 
-
-	/* Close file. */
-	fclose(fp);
-	fp = NULL;
-
-
 	/* Add audio lengths from all data blocks containing audio data. */
 	vd->total_data_len = 0;
 	for(x = 0; x < vd->total_datablocks; x++)
@@ -653,12 +658,26 @@ int VocReadHeader(char *filename, voc_data_struct *vd)
 	}
 
 
+        /* Transfer given fp to the vd structure, the calling
+         * function should not referance it again since we are returning
+         * VocSuccess.
+         */
+        vd->fp = fp;  
 
 	return(VocSuccess);
 }
 
-
-
+/*
+ *	Reads a segment of data from the VOC file reffered to
+ *      by the filename member on the given wd structure (which
+ *      should have been initialized wuth a prior call to
+ *      VocReadHeader().
+ *
+ *      File will be opened as needed if the fp member is NULL.
+ *
+ *      The member data and data_len will be reallocated as needed
+ *      if the requested max_chunk_size is different than data_len.
+ */
 int VocReadPartialData(
         voc_data_struct *vd,
         off_t offset,
@@ -666,131 +685,122 @@ int VocReadPartialData(
         int read_opt		/* Reading format. */
 )
 {
-	unsigned char type;
-        int i;
-
-	FILE *fp;
-
-	int datablock_num;
+        int i, datablock_num;
 	int total_datablocks;
+	voc_block_struct *block_ptr;
 
 	off_t src_buf_pos;
 	int tar_buf_pos;
 
-        off_t filesize;
 	off_t offset_count;
 
 
-        /* Error checks. */
         if(vd == NULL)
             return(VocErrorBadValue);
-	if(vd->filename == NULL)
-	    return(VocErrorBadValue); 
+        if(offset < 0)
+            return(VocErrorBadValue);
 
-
-	/* Zero chunk size means free data. */
-        if(max_chunk_size == 0)
+	/* Read nothing? */
+        if(max_chunk_size <= 0)
 	{
-	    free(vd->data); vd->data = NULL;
+	    free(vd->data);
+	    vd->data = NULL;
 	    vd->data_len = 0;
 
             return(VocSuccess);
 	}
 
-
 	/* Get total_datablocks. */
 	total_datablocks = vd->total_datablocks;
 
-	/* Free and allocate data buffer. */
-        free(vd->data);
-        vd->data_len = max_chunk_size;
-	vd->data = (char *)calloc(
-	    1,
-	    vd->data_len * sizeof(char)
-	);
+	/* Free and allocate data buffer as needed. */
+	if(vd->data_len != max_chunk_size)
+	{
+	    vd->data_len = max_chunk_size;
+
+	    vd->data = (char *)realloc(
+		vd->data,
+		vd->data_len * sizeof(char)
+	    );
+	}
 	if(vd->data == NULL)
 	{
 	    vd->data_len = 0;
 	    return(VocErrorNoBuffers);
 	}
 
+        /* Open file as needed. */
+        if(vd->fp == NULL)
+        {
+            off_t filesize;
 
-        /* Open the file. */
-        fp = VocOpenFile(vd->filename, "rb", &filesize);
-        if(fp == NULL)
-            return(VocErrorNoAccess);
-        if(filesize == 0)
-            return(VocErrorNoAccess);
-        
+            if(vd->filename == NULL)
+                return(VocErrorBadValue);
+         
+            vd->fp = VocOpenFile(vd->filename, "rb", &filesize);
+            if(vd->fp == NULL)
+                return(VocErrorNoAccess);
+            if(filesize == 0)
+                return(VocErrorNoAccess);
+        }
 
-	tar_buf_pos = 0;
-	datablock_num = 0;
-	offset_count = 0;
-	while((datablock_num < total_datablocks) &&
-              (tar_buf_pos < max_chunk_size)
+	/* Go through each data block. */
+	for(datablock_num = 0, tar_buf_pos = 0, offset_count = 0;
+	    datablock_num < total_datablocks;
+	    datablock_num++
 	)
 	{
-	    /* Skip unallocated datablocks. */
-	    if(vd->datablock[datablock_num] == NULL)
-	    {
-		datablock_num++;
-		continue;
-	    }
+	    if(tar_buf_pos >= max_chunk_size)
+		break;
 
-	    /* Read only datablock types audio and audio continue. */
-	    type = vd->datablock[datablock_num]->type;
-	    if((type != VocBlockTypeSoundData) &&
-	       (type != VocBlockTypeSoundDataContinue)
+	    block_ptr = vd->datablock[datablock_num];
+	    if(block_ptr == NULL)
+		continue;
+
+	    /* Skip data blocks not of type audio or audio continue. */
+	    if((block_ptr->type != VocBlockTypeSoundData) &&
+	       (block_ptr->type != VocBlockTypeSoundDataContinue)
 	    )
-            {
-                datablock_num++;
                 continue;
-            }
 
 	    /* Skip if offset would not put us in this block. */
-	    if((offset_count + vd->datablock[datablock_num]->data_len)
-		< offset)
+	    if((offset_count + block_ptr->data_len) < offset)
 	    {
-		offset_count += vd->datablock[datablock_num]->data_len;
-                datablock_num++;
+		offset_count += block_ptr->data_len;
 		continue;
 	    }
 
-	    /* Adjust file pointer. */
+	    /* Seek beginning of audio data block in file. */
 	    src_buf_pos = (off_t)((int)offset - (int)offset_count);
+	    if(src_buf_pos < 0)
+		src_buf_pos = 0;
 	    fseek(
-		fp,
-		vd->datablock[datablock_num]->data_start_pos + src_buf_pos,
+		vd->fp,
+		block_ptr->data_start_pos + src_buf_pos,
 		SEEK_SET
 	    );
 
-	    offset_count += vd->datablock[datablock_num]->data_len;
+	    offset_count += block_ptr->data_len;
 	    offset = offset_count;
 	    while((tar_buf_pos < max_chunk_size) &&
-                  (src_buf_pos < vd->datablock[datablock_num]->data_len)
+                  (src_buf_pos < block_ptr->data_len)
 	    )
 	    {
-		vd->data[tar_buf_pos] = (char)fgetc(fp);
+		vd->data[tar_buf_pos] = (char)fgetc(vd->fp);
 
 		tar_buf_pos++;
 		src_buf_pos++;
 	    }
-
-	    datablock_num++;
 	}
 
 
-	/* Limit data_len. */
+	/* Sanitize data length. */
 	if(tar_buf_pos < vd->data_len)
 	    vd->data_len = tar_buf_pos;
 
 
-	fclose(fp);
-	fp = NULL;
-
-
-        /*   Shift the DSP, voc file format stores the values
-	 *   as u_int8_t.
+        /* Shift the DSP data that was read, voc file format stores the
+	 * data as unsigned char.
 	 */
         switch(read_opt)
         {
@@ -803,8 +813,5 @@ int VocReadPartialData(
             break;
         }
 
-
-
-
 	return(VocSuccess);
 }
