@@ -6,7 +6,6 @@
 #define YHOST_H
 
 #include <stdio.h>
-#include <db.h>
 #include <sys/types.h>
 #include "ytypes.h"
 
@@ -35,7 +34,7 @@ extern int YHostDeleteByHost(YIPUnion *ip);
 
 extern int YHostAllocate(YIPUnion *ip);
 extern void YHostDelete(int n);
-extern void YHostDeleteAll();
+extern void YHostDeleteAll(void);
 
 
 #endif	/* YHOST_H */
