@@ -1,27 +1,5 @@
-/*
-                         Mixer Channel Device IO
-
-	Functions:
-
-        int YMixerGet(
-                Recorder *recorder,
-                int mixer_code,
-                Coefficient *value1,
-                Coefficient *value2
-        )
-
-	int MYixerSet(
-	        Recorder *recorder,
-	        int mixer_code,
-	        Coefficient value1,
-	        Coefficient value2
-	)
-
-	---
-
- */
-
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -33,55 +11,65 @@
 
 #include "ysound.h"
 #include "ymixer.h"
+#include "options.h"
 
 
+static int GET_MIXER_NUM_BY_NAME(int fd, const char *name);
 
+static int YMixerSetBass(int fd, double value1, double value2);
+static int YMixerSetCD(int fd, double value1, double value2);
+static int YMixerSetGainIn(int fd, double value1, double value2);
+static int YMixerSetGainOut(int fd, double value1, double value2);
+static int YMixerSetLine(int fd, double value1, double value2);
+static int YMixerSetLine1(int fd, double value1, double value2);
+static int YMixerSetLine2(int fd, double value1, double value2);
+static int YMixerSetLine3(int fd, double value1, double value2);
+static int YMixerSetMic(int fd, double value1, double value2);
+static int YMixerSetMix(int fd, double value1, double value2);
+static int YMixerSetPCM(int fd, double value1, double value2);
+static int YMixerSetPCM2(int fd, double value1, double value2);
+static int YMixerSetRec(int fd, double value1, double value2);
+static int YMixerSetSpeaker(int fd, double value1, double value2);
+static int YMixerSetSynth(int fd, double value1, double value2);
+static int YMixerSetTreble(int fd, double value1, double value2);
+static int YMixerSetVolume(int fd, double value1, double value2);
 
-int GET_MIXER_NUM_BY_NAME(int fd, char *name);
+static int YMixerGetBass(int fd, double *value, double *value2);
+static int YMixerGetCD(int fd, double *value1, double *value2);
+static int YMixerGetGainIn(int fd, double *value1, double *value2);
+static int YMixerGetGainOut(int fd, double *value1, double *value2);
+static int YMixerGetLine(int fd, double *value, double *value2);
+static int YMixerGetLine1(int fd, double *value, double *value2);
+static int YMixerGetLine2(int fd, double *value, double *value2);
+static int YMixerGetLine3(int fd, double *value, double *value2);
+static int YMixerGetMic(int fd, double *value1, double *value2);
+static int YMixerGetMix(int fd, double *value, double *value2);
+static int YMixerGetPCM(int fd, double *value1, double *value2);
+static int YMixerGetPCM2(int fd, double *value1, double *value2);
+static int YMixerGetRec(int fd, double *value, double *value2);
+static int YMixerGetSpeaker(int fd, double *value, double *value2);
+static int YMixerGetSynth(int fd, double *value1, double *value2);
+static int YMixerGetTreble(int fd, double *value, double *value2);
+static int YMixerGetVolume(int fd, double *value1, double *value2);
 
+int YMixerGet(
+        Recorder *recorder,
+        int mixer_code,
+        Coefficient *value
+);
 
-int YMixerSetBass(int fd, double value1, double value2);
-int YMixerSetCD(int fd, double value1, double value2);
-int YMixerSetGainIn(int fd, double value1, double value2);
-int YMixerSetGainOut(int fd, double value1, double value2);
-int YMixerSetLine(int fd, double value1, double value2);
-int YMixerSetLine1(int fd, double value1, double value2);
-int YMixerSetLine2(int fd, double value1, double value2);
-int YMixerSetLine3(int fd, double value1, double value2);
-int YMixerSetMic(int fd, double value1, double value2);
-int YMixerSetMix(int fd, double value1, double value2);
-int YMixerSetPCM(int fd, double value1, double value2);
-int YMixerSetPCM2(int fd, double value1, double value2);
-int YMixerSetRec(int fd, double value1, double value2);
-int YMixerSetSpeaker(int fd, double value1, double value2);
-int YMixerSetSynth(int fd, double value1, double value2);
-int YMixerSetTreble(int fd, double value1, double value2);
-int YMixerSetVolume(int fd, double value1, double value2);
-
-int YMixerGetBass(int fd, double *value, double *value2);
-int YMixerGetCD(int fd, double *value1, double *value2);
-int YMixerGetGainIn(int fd, double *value1, double *value2);
-int YMixerGetGainOut(int fd, double *value1, double *value2);
-int YMixerGetLine(int fd, double *value, double *value2);
-int YMixerGetLine1(int fd, double *value, double *value2);
-int YMixerGetLine2(int fd, double *value, double *value2);
-int YMixerGetLine3(int fd, double *value, double *value2);
-int YMixerGetMic(int fd, double *value1, double *value2);
-int YMixerGetMix(int fd, double *value, double *value2);
-int YMixerGetPCM(int fd, double *value1, double *value2);
-int YMixerGetPCM2(int fd, double *value1, double *value2);
-int YMixerGetRec(int fd, double *value, double *value2);
-int YMixerGetSpeaker(int fd, double *value, double *value2);
-int YMixerGetSynth(int fd, double *value1, double *value2);
-int YMixerGetTreble(int fd, double *value, double *value2);
-int YMixerGetVolume(int fd, double *value1, double *value2);
+int YMixerSet(
+        Recorder *recorder,
+        int mixer_code,         /* One of YMixerCode*. */
+        Coefficient *value      /* Each value from 0.0 to 1.0. */
+);
 
 
 /*
  *	Returns the mixer device's number matching name or
  *	-1 on failed match.
  */
-int GET_MIXER_NUM_BY_NAME(int fd, char *name)
+static int GET_MIXER_NUM_BY_NAME(int fd, const char *name)
 {
 	int i;
 	int mixermask;
@@ -116,32 +104,35 @@ int GET_MIXER_NUM_BY_NAME(int fd, char *name)
 } 
 
 
-
-/* **************************************************************** */
-
+/*
+ *	Macro to set the new mixer value. Requires variables;
+ *	fd, stereomask, value1, value2, and i.
+ */
 #define YMIXER_SET_MAC	\
-if(ioctl(fd, SOUND_MIXER_READ_STEREODEVS, &stereomask))\
- return(-1);\
-\
-/* Is this a stereo channel device? */\
-if((value1 != value2) && !((1 << i) & stereomask))\
-{\
- value1 = (value1 + value2) / 2;\
- value2 = value1;\
-}\
-\
-left = (unsigned int)(value1 * 100);\
-right = (unsigned int)(value2 * 100);\
-\
-packed_value = (right << 8) + left;\
-\
-return(ioctl(fd, MIXER_WRITE(i), &packed_value));
+{ \
+ if(ioctl(fd, SOUND_MIXER_READ_STEREODEVS, &stereomask)) \
+  return(-1); \
+ \
+ /* Is this a stereo channel device? */ \
+ if((value1 != value2) && !((1 << i) & stereomask)) \
+ { \
+  value1 = (value1 + value2) / 2; \
+  value2 = value1; \
+ } \
+ \
+ left = (unsigned int)(value1 * 100); \
+ right = (unsigned int)(value2 * 100); \
+ \
+ packed_value = (right << 8) + left; \
+ \
+ return(ioctl(fd, MIXER_WRITE(i), &packed_value)); \
+}
 
 
 /*
  *	Mixer channel device set functions.
  */
-int YMixerSetBass(int fd, double value1, double value2)
+static int YMixerSetBass(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -153,7 +144,7 @@ int YMixerSetBass(int fd, double value1, double value2)
 	YMIXER_SET_MAC
 }
 
-int YMixerSetCD(int fd, double value1, double value2)
+static int YMixerSetCD(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -165,7 +156,7 @@ int YMixerSetCD(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetGainIn(int fd, double value1, double value2)
+static int YMixerSetGainIn(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -177,7 +168,7 @@ int YMixerSetGainIn(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetGainOut(int fd, double value1, double value2)
+static int YMixerSetGainOut(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -189,7 +180,7 @@ int YMixerSetGainOut(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetLine(int fd, double value1, double value2)
+static int YMixerSetLine(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -201,7 +192,7 @@ int YMixerSetLine(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetLine1(int fd, double value1, double value2)
+static int YMixerSetLine1(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -213,7 +204,7 @@ int YMixerSetLine1(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetLine2(int fd, double value1, double value2)
+static int YMixerSetLine2(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -225,7 +216,7 @@ int YMixerSetLine2(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetLine3(int fd, double value1, double value2)
+static int YMixerSetLine3(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
         
@@ -237,7 +228,7 @@ int YMixerSetLine3(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetMic(int fd, double value1, double value2)
+static int YMixerSetMic(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -249,7 +240,7 @@ int YMixerSetMic(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetMix(int fd, double value1, double value2)
+static int YMixerSetMix(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -261,7 +252,7 @@ int YMixerSetMix(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetPCM(int fd, double value1, double value2)
+static int YMixerSetPCM(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -273,7 +264,7 @@ int YMixerSetPCM(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetPCM2(int fd, double value1, double value2)
+static int YMixerSetPCM2(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -285,7 +276,7 @@ int YMixerSetPCM2(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetRec(int fd, double value1, double value2)
+static int YMixerSetRec(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -297,7 +288,7 @@ int YMixerSetRec(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetSpeaker(int fd, double value1, double value2)
+static int YMixerSetSpeaker(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
         
@@ -309,7 +300,7 @@ int YMixerSetSpeaker(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetSynth(int fd, double value1, double value2)
+static int YMixerSetSynth(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -321,7 +312,7 @@ int YMixerSetSynth(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetTreble(int fd, double value1, double value2)
+static int YMixerSetTreble(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -333,7 +324,7 @@ int YMixerSetTreble(int fd, double value1, double value2)
         YMIXER_SET_MAC
 }
 
-int YMixerSetVolume(int fd, double value1, double value2)
+static int YMixerSetVolume(int fd, double value1, double value2)
 {
         int i, left, right, stereomask, packed_value;
 
@@ -346,20 +337,22 @@ int YMixerSetVolume(int fd, double value1, double value2)
 }
 
 
-/* ***************************************************************** */
-
+/*
+ *	Macro to get mixer values from mixer channel i. Uses variables;
+ *	fd, packed_value, i, value1, and value2.
+ */
 #define YMIXER_GET_MAC	\
-if(ioctl(fd, MIXER_READ(i), &packed_value))\
- return(-1);\
-*value1 = (double)(packed_value & 0x000000ff) / (double)100;\
-*value2 = (double)((packed_value & 0x0000ff00) >> 8) / (double)100;
-
-
+{ \
+ if(ioctl(fd, MIXER_READ(i), &packed_value)) \
+  return(-1); \
+ (*value1) = (double)(packed_value & 0x000000ff) / (double)100; \
+ (*value2) = (double)((packed_value & 0x0000ff00) >> 8) / (double)100; \
+}
 
 /*
  *      Mixer channel device get functions.
  */
-int YMixerGetBass(int fd, double *value1, double *value2)
+static int YMixerGetBass(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -373,7 +366,7 @@ int YMixerGetBass(int fd, double *value1, double *value2)
 	return(0);
 }
 
-int YMixerGetCD(int fd, double *value1, double *value2)
+static int YMixerGetCD(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -387,7 +380,7 @@ int YMixerGetCD(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetGainIn(int fd, double *value1, double *value2)
+static int YMixerGetGainIn(int fd, double *value1, double *value2)
 {
         int i, packed_value;
         
@@ -401,7 +394,7 @@ int YMixerGetGainIn(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetGainOut(int fd, double *value1, double *value2)
+static int YMixerGetGainOut(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -415,7 +408,7 @@ int YMixerGetGainOut(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetLine(int fd, double *value1, double *value2)
+static int YMixerGetLine(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -429,7 +422,7 @@ int YMixerGetLine(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetLine1(int fd, double *value1, double *value2)
+static int YMixerGetLine1(int fd, double *value1, double *value2)
 {
         int i, packed_value;
         
@@ -443,7 +436,7 @@ int YMixerGetLine1(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetLine2(int fd, double *value1, double *value2)
+static int YMixerGetLine2(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -457,7 +450,7 @@ int YMixerGetLine2(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetLine3(int fd, double *value1, double *value2)
+static int YMixerGetLine3(int fd, double *value1, double *value2)
 {
         int i, packed_value;
         
@@ -471,7 +464,7 @@ int YMixerGetLine3(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetMic(int fd, double *value1, double *value2)
+static int YMixerGetMic(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -485,7 +478,7 @@ int YMixerGetMic(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetMix(int fd, double *value1, double *value2)
+static int YMixerGetMix(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -499,7 +492,7 @@ int YMixerGetMix(int fd, double *value1, double *value2)
         return(0);
 }       
 
-int YMixerGetPCM(int fd, double *value1, double *value2)
+static int YMixerGetPCM(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -513,7 +506,7 @@ int YMixerGetPCM(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetPCM2(int fd, double *value1, double *value2)
+static int YMixerGetPCM2(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -527,7 +520,7 @@ int YMixerGetPCM2(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetRec(int fd, double *value1, double *value2)
+static int YMixerGetRec(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -541,7 +534,7 @@ int YMixerGetRec(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetSpeaker(int fd, double *value1, double *value2)
+static int YMixerGetSpeaker(int fd, double *value1, double *value2)
 {
         int i, packed_value;
         
@@ -555,7 +548,7 @@ int YMixerGetSpeaker(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetSynth(int fd, double *value1, double *value2)
+static int YMixerGetSynth(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -569,7 +562,7 @@ int YMixerGetSynth(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetTreble(int fd, double *value1, double *value2)
+static int YMixerGetTreble(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -583,7 +576,7 @@ int YMixerGetTreble(int fd, double *value1, double *value2)
         return(0);
 }
 
-int YMixerGetVolume(int fd, double *value1, double *value2)
+static int YMixerGetVolume(int fd, double *value1, double *value2)
 {
         int i, packed_value;
 
@@ -604,125 +597,147 @@ int YMixerGetVolume(int fd, double *value1, double *value2)
  *      Gets mixer channel device values for the specified channel
  *	device mixer_code.   Returns -1 on error, -2 on unsupported
  *	mixer_code, or 0 on success.
+ *
+ *      The given value must point to an array of YMixerValues many 
+ *      values which will be defined correctly on success.
  */
 int YMixerGet(
 	Recorder *recorder,
 	int mixer_code,		/* One of YMixerCode*. */
-	Coefficient *value1,	/* [0, 1]. */
-	Coefficient *value2	/* [0, 1]. */
+	Coefficient *value
 )
 {
-	int fd, status = -2;
+	int i, fd, status = -2;
+	int mixer_index = mixer_code - YMixerCodeBaseOffset;
 
 
-	if(recorder == NULL)
+	if((recorder == NULL) || (value == NULL))
 	    return(-1);
 
+        /* Reset return values. */
+        for(i = 0; i < YMixerValues; i++)
+	    value[i] = 0.0;
+
+	/* Get mixer file descriptor. */
 	fd = recorder->audio.mixer_fd;
         if(fd < 0)
             return(-1);
 
+	/* Handle by mixer code. */
 
-	if((value1 == NULL) ||
-           (value2 == NULL)
-	)
-	    return(-1);
-
-	*value1 = 0;
-	*value2 = 0;
-
+if(0)
+{
+/* Don't query from device. */
         switch(mixer_code)
         {
           case YMixerCodeBass:
-            status = YMixerGetBass(fd, value1, value2);
+            status = YMixerGetBass(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeCD:
-            status = YMixerGetCD(fd, value1, value2);
+            status = YMixerGetCD(fd, &value[0], &value[1]);
 	    break;
 
           case YMixerCodeGainIn:
-            status = YMixerGetGainIn(fd, value1, value2);
+            status = YMixerGetGainIn(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeGainOut:
-            status = YMixerGetGainOut(fd, value1, value2);
+            status = YMixerGetGainOut(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeLine:
-            status = YMixerGetLine(fd, value1, value2);
+            status = YMixerGetLine(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeLine1:
-            status = YMixerGetLine1(fd, value1, value2);
+            status = YMixerGetLine1(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeLine2:
-            status = YMixerGetLine2(fd, value1, value2);
+            status = YMixerGetLine2(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeLine3:
-            status = YMixerGetLine3(fd, value1, value2);
+            status = YMixerGetLine3(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeMic:
-            status = YMixerGetMic(fd, value1, value2);
+            status = YMixerGetMic(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeMix:
-            status = YMixerGetMix(fd, value1, value2);
+            status = YMixerGetMix(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodePCM:
-            status = YMixerGetPCM(fd, value1, value2);        
+            status = YMixerGetPCM(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodePCM2:
-            status = YMixerGetPCM2(fd, value1, value2);
+            status = YMixerGetPCM2(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeRec:
-            status = YMixerGetRec(fd, value1, value2);
+            status = YMixerGetRec(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeSpeaker:
-            status = YMixerGetSpeaker(fd, value1, value2);
+            status = YMixerGetSpeaker(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeSynth:
-            status = YMixerGetSynth(fd, value1, value2);
+            status = YMixerGetSynth(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeTreble:
-            status = YMixerGetTreble(fd, value1, value2);
+            status = YMixerGetTreble(fd, &value[0], &value[1]);
             break;
 
           case YMixerCodeVolume:
-            status = YMixerGetVolume(fd, value1, value2);
+            status = YMixerGetVolume(fd, &value[0], &value[1]);
             break;
 	}
+} else {
 
+/* For now just get from mixer index. */
+
+        if((mixer_index >= 0) && (mixer_index < YTotalMixers))
+        {
+            Coefficient *opt_mixer_value = option.mixer_value[mixer_index];
+ 
+            for(i = 0; i < YMixerValues; i++)
+                value[i] = opt_mixer_value[i];
+        }
+
+}
 
 	return(0);
 }
 
 
 /*
- *	Sets mixer channel device values for the specified channel
+ *	Sets actual mixer channel device values for the specified channel
  *	device mixer_code. Returns -1 on error, -2 on unsupported
  *	mixer_code, or 0 on success.
+ *
+ *	The given value must point to an array of YMixerValues many
+ *	values.
+ *
+ *	The recorder's audio structure will be updated with the newly set
+ *	mixer value on success.
  */
 int YMixerSet(
-	Recorder *recorder,
-	int mixer_code,		/* One of YMixerCode*. */
-	Coefficient value1,	/* [0, 1]. */
-	Coefficient value2	/* [0, 1]. */
+        Recorder *recorder,
+        int mixer_code,         /* One of YMixerCode*. */
+        Coefficient *value      /* Each value from 0.0 to 1.0. */
 )
 {
-	int fd, status = -2;
+	int i, fd, status = -2;
+	int mixer_index = mixer_code - YMixerCodeBaseOffset;
 
 
-        if(recorder == NULL)
+        if((recorder == NULL) || (value == NULL))
             return(-1);
 
         fd = recorder->audio.mixer_fd;
@@ -730,89 +745,95 @@ int YMixerSet(
             return(-1);
 
 
-	/* Sanitize values. */
-        if(value1 < 0)
-            value1 = 0;
-        if(value1 > 1)
-            value1 = 1;
+	/* Clip. */
+	for(i = 0; i < YMixerValues; i++)
+	{
+	    if(value[i] < 0.0)
+		value[i] = 0.0;
+	    if(value[i] > 1.0)
+		value[i] = 1.0;
+	}
 
-        if(value2 < 0)
-            value2 = 0;
-        if(value2 > 1)
-            value2 = 1;
-        
+	/* Update last mixer value on recorder's audio structure. */
+	if((mixer_index >= 0) && (mixer_index < YTotalMixers))
+	{
+	    Coefficient *opt_mixer_value = option.mixer_value[mixer_index];
+
+	    for(i = 0; i < YMixerValues; i++)
+		opt_mixer_value[i] = value[i];
+	}
+
+	/* Handle actual setting by mixer code. */
 	switch(mixer_code)
 	{
 	  case YMixerCodeBass:
-            status = YMixerSetBass(fd, value1, value2);
+            status = YMixerSetBass(fd, value[0], value[1]);
             break;
 
           case YMixerCodeCD:
-            status = YMixerSetCD(fd, value1, value2);
+            status = YMixerSetCD(fd, value[0], value[1]);
             break;
 
           case YMixerCodeGainIn:
-            status = YMixerSetGainIn(fd, value1, value2);
+            status = YMixerSetGainIn(fd, value[0], value[1]);
             break;
 
           case YMixerCodeGainOut:
-            status = YMixerSetGainOut(fd, value1, value2);
+            status = YMixerSetGainOut(fd, value[0], value[1]);
             break;
 
           case YMixerCodeLine:
-            status = YMixerSetLine(fd, value1, value2);
+            status = YMixerSetLine(fd, value[0], value[1]);
             break;
 
           case YMixerCodeLine1:
-            status = YMixerSetLine1(fd, value1, value2);
+            status = YMixerSetLine1(fd, value[0], value[1]);
             break;
 
           case YMixerCodeLine2:
-            status = YMixerSetLine2(fd, value1, value2);
+            status = YMixerSetLine2(fd, value[0], value[1]);
             break;
 
           case YMixerCodeLine3:
-            status = YMixerSetLine3(fd, value1, value2);
+            status = YMixerSetLine3(fd, value[0], value[1]);
             break;
 
           case YMixerCodeMic:
-            status = YMixerSetMic(fd, value1, value2);
+            status = YMixerSetMic(fd, value[0], value[1]);
             break;
 
           case YMixerCodeMix:
-            status = YMixerSetMix(fd, value1, value2);
+            status = YMixerSetMix(fd, value[0], value[1]);
             break;
 
           case YMixerCodePCM:
-            status = YMixerSetPCM(fd, value1, value2);
+            status = YMixerSetPCM(fd, value[0], value[1]);
             break;
 
           case YMixerCodePCM2:
-            status = YMixerSetPCM2(fd, value1, value2);
+            status = YMixerSetPCM2(fd, value[0], value[1]);
             break;
 
           case YMixerCodeRec:
-            status = YMixerSetRec(fd, value1, value2);
+            status = YMixerSetRec(fd, value[0], value[1]);
             break;
 
           case YMixerCodeSpeaker:
-            status = YMixerSetSpeaker(fd, value1, value2);
+            status = YMixerSetSpeaker(fd, value[0], value[1]);
             break;
 
           case YMixerCodeSynth:
-            status = YMixerSetSynth(fd, value1, value2);
+            status = YMixerSetSynth(fd, value[0], value[1]);
             break;
 
           case YMixerCodeTreble:
-            status = YMixerSetTreble(fd, value1, value2);
+            status = YMixerSetTreble(fd, value[0], value[1]);
             break;
 
 	  case YMixerCodeVolume:
-	    status = YMixerSetVolume(fd, value1, value2);  
+	    status = YMixerSetVolume(fd, value[0], value[1]);
 	    break;
 	}
 
-
-
         return(status);
 }
