@@ -10,22 +10,19 @@
 
 #include "ytypes.h"
 #include "ysound.h"
-#include "../include/Ymixercodes.h"
+#include "../include/Y2/Ymixercodes.h"
 
 
 extern int YMixerGet(
         Recorder *recorder,
         int mixer_code,		/* One of YMixerCode*. */
-        Coefficient *value1,	/* [0, 1]. */
-        Coefficient *value2	/* [0, 1]. */
+        Coefficient *value
 );
 
 extern int YMixerSet(
         Recorder *recorder,
         int mixer_code,		/* One of YMixerCode*. */
-        Coefficient value1,	/* [0, 1]. */
-        Coefficient value2	/* [0, 1]. */
+	Coefficient *value	/* Each value from 0.0 to 1.0. */
 );
 
-
 #endif	/* YMIXER_H */
