@@ -8,7 +8,7 @@
 #ifndef YMODES_H
 #define YMODES_H
 
-#include "ytimming.h"
+#include "ytiming.h"
 #include "ytypes.h"
 
 
@@ -45,14 +45,13 @@ extern int total_ymodes;
  */
 extern int YModeIsAllocated(int n);
 extern YMode *YModeGetPtr(int n);
-extern int YModeMatch(char *name);
+extern int YModeMatch(const char *name);
 
-extern int YModeAllocate(char *name);
+extern int YModeAllocate(const char *name);
 extern void YModeReset(int n);
 extern void YModeDelete(int n);
-
-extern void YModeDeleteAll();
-extern void YModeReclaim();
+extern void YModeDeleteAll(void);
+extern void YModeReclaim(void);
 
 
 #endif	/* YMODES_H */
