@@ -1,135 +1,12 @@
-/*
-
-	Functions:
-
-	void YNetPrintError(
-	        FILE *stream,
-	        YConnectionNumber con_num,
-	        u_int32_t chunk_length,
-	        u_int16_t major_op_code,
-	        u_int16_t minor_op_code,
-	        char *mesg
-	)
-
-	int YNetSendToConnection(
-		YConnectionNumber con_num,
-		char *buf,
-		int len
-	)
-
-	int YNetSendAudioChangePreset(
-		YConnectionNumber con_num,
-		char *audio_mode_name
-	)
-	int YNetSendAudioChangeValues(
-	        YConnectionNumber con_num,
-	        int sample_size,
-	        int channels,
-	        YDataLength sample_rate,
-	        int direction,
-	        int allow_fragmenting,
-	        int num_fragments,
-	        int fragment_size
-	)
-	int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendServerStats(
-		YConnectionNumber con_num,
-		int protocol_version_major,
-		int protocol_version_minor,
-		Coefficient cycle_load,
-		char *vendor_name
-	)
-	int YNetParseServerStats(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendCycleChange(YConnectionNumber con_num, long cycle_us)
-	int YNetParseCycleChange(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendDisconnect(YConnectionNumber con_num, u_int16_t reason)
-	int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSetHost(
-	        YConnectionNumber con_num,
-	        u_int16_t minor_op_code,
-	        YIPUnion *ip
-	)        
-	int YNetParseSetHost(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSetMixerDevice(
-	        YConnectionNumber con_num,
-	        int mixer_code,
-	        Coefficient value1,
-	        Coefficient value2
-	)
-	int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSoundObjectPlay(
-	        YConnectionNumber con_num,
-	        YID yid,
-	        char *path,
-	        YDataPosition pos,
-	        YVolumeStruct *volume,
-	        int repeats
-	)
-	int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSoundObjectKill(YConnectionNumber con_num, YID yid)
-	int YNetParseSoundObjectKill(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSoundObjectAttributes(
-		YConnectionNumber con_num,
-                char *path,
-		int format,
-                int sample_size,
-		int channels,
-		int sample_rate
-	)
-	int YNetParseSoundObjectAttributes(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendShutdown(YConnectionNumber con_num, u_int16_t reason)
-	int YNetParseShutdown(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendSync(YConnectionNumber con_num, long us)
-	int YNetParseSync(YCNP_STD_INPUTS_PROTO)
-
-        int YNetSendAudioStats(YConnectionNumber con_num)
-        int YNetParseAudioStats(YCNP_STD_INPUTS_PROTO)
-
-	int YNetSendListAudioModes(
-	        YConnectionNumber con_num,
-	        char *audio_mode_name,
-	        int sample_rate, int channels,
-	        int sample_size, int fragment_size_bytes,
-	        char direction, Boolean allow_fragmenting, int num_fragments 
-	)
-	int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO)
-
-	int YNetParse(
-	        YConnectionNumber con_num,
-	        u_int8_t *buf,
-	        u_int32_t chunk_length,
-	        u_int16_t major_op_code,
-	        u_int16_t minor_op_code
-	)
-
-	int YNetRecv(YConnectionNumber con_num);
-
-
-
-	---
-
- */
-
 #include <stdio.h>
-#include <db.h>
-#include <sys/types.h> 
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
+#include <sys/types.h> 
 #include <errno.h>
 extern int errno;
 #include <unistd.h>
 #include <signal.h>
-#include <sys/time.h>
+#include <math.h>
 #include <sys/stat.h>
 
 #include <netinet/in.h>
@@ -138,6 +15,8 @@ extern int errno;
 #include <sys/ioctl.h>
 #include <fcntl.h>
 
+#include "../include/string.h"
+
 #include "ytypes.h"
 #include "soundpaths.h"
 #include "ysound.h"
@@ -152,11 +31,131 @@ extern int errno;
 #include "options.h"
 
 
+void YNetPrintError(
+	FILE *stream,
+	YConnectionNumber con_num,
+	u_int32_t chunk_length,
+	u_int16_t major_op_code,
+	u_int16_t minor_op_code,
+	const char *mesg
+);
+
+int YNetSendToConnection(
+	YConnectionNumber con_num, const char *buf, int len
+);
+
+int YNetSendAudioChangePreset(
+	YConnectionNumber con_num,
+	const char *audio_mode_name
+);
+int YNetSendAudioChangeValues(
+	YConnectionNumber con_num,
+	int sample_size,
+	int channels,
+	YDataLength sample_rate,
+	int direction,
+	int allow_fragmenting,
+	int num_fragments,
+	int fragment_size
+);
+
+int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendServerStats(
+	YConnectionNumber con_num,
+	int protocol_version_major,
+	int protocol_version_minor,
+	Coefficient cycle_load,
+	const char *vendor_name
+);
+int YNetParseServerStats(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendCycleChange(YConnectionNumber con_num, long cycle_us);
+int YNetParseCycleChange(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendDisconnect(YConnectionNumber con_num, u_int16_t reason);
+int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSetHost(
+	YConnectionNumber con_num,
+	u_int16_t minor_op_code,
+	const YIPUnion *ip
+);
+int YNetParseSetHost(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSetMixerDevice(
+	YConnectionNumber con_num,
+	int mixer_code,
+	Coefficient value1, Coefficient value2
+);
+int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSoundObjectPlay(
+	YConnectionNumber con_num,
+	YID yid,
+	const char *path,
+	YDataPosition pos,
+	YVolumeStruct *volume,
+	int sample_rate,
+	int repeats
+);
+int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSoundObjectKill(YConnectionNumber con_num, YID yid);
+int YNetParseSoundObjectKill(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSoundObjectAttributes(
+	YConnectionNumber con_num,
+	const char *path,
+	int format,
+	int sample_size,
+	int channels,
+	int sample_rate
+);
+int YNetParseSoundObjectAttributes(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendShutdown(YConnectionNumber con_num, u_int16_t reason);
+int YNetParseShutdown(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendSync(YConnectionNumber con_num, long us);
+int YNetParseSync(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendAudioStats(
+        YConnectionNumber con_num,
+        Audio *audio
+);
+int YNetParseAudioStats(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendListAudioModes(
+	YConnectionNumber con_num,
+	const char *audio_mode_name,
+	int sample_rate, int channels,
+	int sample_size, int fragment_size_bytes,
+	char direction, Boolean allow_fragmenting, int num_fragments
+);
+int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO);
+
+int YNetSendPlaySoundObjectValues(
+	YConnectionNumber con_num,
+	PlayStack *ps_ptr
+);
+int YNetParsePlaySoundObjectValues(YCNP_STD_INPUTS_PROTO);
+
+int YNetParse(
+	YConnectionNumber con_num,
+	const u_int8_t *buf,
+	u_int32_t chunk_length,
+        u_int16_t major_op_code,
+        u_int16_t minor_op_code
+);
+int YNetRecv(YConnectionNumber con_num);;
+
+
 #define MIN(a,b)	((a) < (b) ? (a) : (b))
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
 
-extern int antishift(int in);   /* In main.c */
+extern int YAntiShift(int in);   /* In main.c */
 
 
 /*
@@ -174,7 +173,7 @@ void YNetPrintError(
         u_int32_t chunk_length,
         u_int16_t major_op_code,
         u_int16_t minor_op_code,
-        char *mesg
+        const char *mesg
 )
 {
         if(stream == NULL)
@@ -218,7 +217,7 @@ void YNetPrintError(
  *	available for sending data. Returns -1 if it was not
  *	able to send the buffer or the number of bytes sent.
  */
-int YNetSendToConnection(YConnectionNumber con_num, char *buf, int len) 
+int YNetSendToConnection(YConnectionNumber con_num, const char *buf, int len) 
 {
 	int s;
         struct timeval timeout;
@@ -256,7 +255,7 @@ int YNetSendToConnection(YConnectionNumber con_num, char *buf, int len)
  */
 int YNetSendAudioChangePreset(
 	YConnectionNumber con_num,
-	char *audio_mode_name
+	const char *audio_mode_name
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
@@ -340,10 +339,9 @@ int YNetSendAudioChangeValues(
 int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
 {
 	int i;
-	PlayStack **ps_ptr;
 	YConnection **yc_ptr;
 	YMode *ymode_ptr;
-
+	Audio *audio_ptr;
 
 	/* Change mode values or change to preset mode? */
         if(minor_op_code == YAudioChangeValues)
@@ -401,6 +399,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                     fragment_size_bytes
                 );
 
+		/* Return now, do not delete playstacks or modify Audio. */
 		return(0);
 	    }
 
@@ -418,26 +417,27 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
             recorder = NULL;
 
 
-            /* Set new Audio values to option's Audio values. */
-            option.audio.sample_size = sample_size;
-            option.audio.channels = channels;
-            option.audio.sample_rate = sample_rate;
+            /* Set new Audio values to option Audio values. */
+	    audio_ptr = &option.audio;
+            audio_ptr->sample_size = sample_size;
+            audio_ptr->channels = channels;
+            audio_ptr->sample_rate = sample_rate;
 
 #ifdef OSS_BUFFRAG
-            option.audio.allow_fragments = ((allow_fragmenting) ?
+            audio_ptr->allow_fragments = ((allow_fragmenting) ?
 		True : False
 	    );
-            option.audio.num_fragments = num_fragments;
-            option.audio.fragment_size = antishift(fragment_size_bytes - 1);
+            audio_ptr->num_fragments = num_fragments;
+            audio_ptr->fragment_size = YAntiShift(fragment_size_bytes - 1);
 #endif  /* OSS_BUFFRAG */
 
-            option.audio.direction = direction;
+            audio_ptr->direction = direction;
 
 
             /* Allocate recorder structure. */
             recorder = (Recorder *)calloc(1, sizeof(Recorder));
             if(recorder == NULL)
-                return;
+                return(-1);
 	    /* Initialize recorder. */
             if(YSoundInit(recorder, &option.audio))
 	    {
@@ -457,7 +457,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                     0, 0, 0, 0
                 );
 
-		return;
+		return(-1);
 	    }
 
             /*  Need to sync audio device right after
@@ -492,7 +492,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
         }
         else
         {
-            /* Parse change Audio mode. */
+            /* Parse change preset Audio mode. */
 
             /*   <u32_cs> <u16_majop> <u16_minop>
              *   <...modename...>
@@ -500,6 +500,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
             const YDataLength base_seg_len = 8;
             int name_len;
 	    int mode_num;
+	    int mode_name_different = 0;
 	    char mode_name[YAudioNameMax];
 	    Audio *audio_ptr;
 
@@ -520,11 +521,40 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                 );
                 mode_name[name_len] = '\0';
 
+		/* Check if the given Audio mode name is different from
+		 * the one specified in the Audio structure.
+		 */
+		if(recorder->audio.audio_mode_name != NULL)
+		{
+		    if(strcasecmp(
+			recorder->audio.audio_mode_name,
+			mode_name
+		    ))
+		        mode_name_different = 1;
+		}
+		else
+		{
+		    /* Last Audio mode name not recorded on Audio
+		     * structure, so assume its different.
+		     */
+		    mode_name_different = 1;
+		}
+		/* Mode name should always be considered different if
+		 * Audio is currently shelled out.
+		 */
+		if(recorder->audio.shelled_out)
+		    mode_name_different = 1;
+
+
+		/* Match preset Audio mode, skip if current Audio
+		 * mode matches.
+		 */
                 mode_num = YModeMatch(mode_name);
-                if(YModeIsAllocated(mode_num))
+                if(YModeIsAllocated(mode_num) &&
+                   mode_name_different
+		)
                 {
 		    ymode_ptr = ymode[mode_num];
-		    audio_ptr = &option.audio;
 
 		    /* Delete all playstacks and notify their respective
 		     * owners about the deletion.
@@ -540,9 +570,8 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
 		    recorder = NULL;
 
 
-                    /* Set option's Audio values to new Audio mode's
-		     * values.
-		     */
+                    /* Set option Audio values to new mode's values. */
+		    audio_ptr = &option.audio;
 
 		    /* Audio modes are always considered user set. */
                     audio_ptr->cycle_set = CYCLE_SET_USER;
@@ -550,6 +579,8 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                     audio_ptr->cycle.ms = ymode_ptr->cycle.ms;
                     audio_ptr->cycle.us = ymode_ptr->cycle.us;
 
+		    /* Leave audio_ptr->compensated_cycle alone. */
+
                     audio_ptr->write_ahead.ms = ymode_ptr->write_ahead.ms;
                     audio_ptr->write_ahead.us = ymode_ptr->write_ahead.us;
 
@@ -570,7 +601,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                     /* Allocate recorder structure. */
                     recorder = (Recorder *)calloc(1, sizeof(Recorder));
                     if(recorder == NULL)
-                        return;
+                        return(-1);
 		    /* Initialize recorder. */
                     if(YSoundInit(recorder, audio_ptr))
 		    {
@@ -588,7 +619,7 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
 			 */
                         YNetSendAudioChangePreset(con_num, "");
 
-			return;
+			return(-1);
 		    }
 
                     /*  Need to sync audio device right after
@@ -596,6 +627,14 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
 		     */
                     YSoundSync(recorder, 0);
 
+		    /* Record name of preset Audio mode used to
+		     * set this Audio.
+		     */
+		    free(recorder->audio.audio_mode_name);
+		    recorder->audio.audio_mode_name = StringCopyAlloc(
+			ymode_ptr->name
+		    );
+
                     /* Notify all connections of Audio mode change. */
                     for(i = 0, yc_ptr = yconnection;
                         i < total_yconnections;
@@ -614,9 +653,15 @@ int YNetParseAudioChange(YCNP_STD_INPUTS_PROTO)
                         );
                     }
 		}
+		/* Audio mode the same? */
+		else if(!mode_name_different)
+		{
+		    /* Send back fake success response. */
+		    YNetSendAudioChangePreset(con_num, mode_name);
+		}
 		else
 		{
-		    /* No such mode. */
+		    /* No such preset Audio mode. */
 
                     /* Notify connection of Audio mode change
 		     * failure (send back empty Audio mode name).
@@ -647,7 +692,7 @@ int YNetSendServerStats(
 	int protocol_version_major,
 	int protocol_version_minor,
 	Coefficient cycle_load,
-	char *vendor_name
+	const char *vendor_name
 )
 {
 	/*   <u32_cs> <u16_majop> <u16_minop>
@@ -892,7 +937,7 @@ int YNetParseDisconnect(YCNP_STD_INPUTS_PROTO)
 int YNetSendSetHost(
 	YConnectionNumber con_num,
 	u_int16_t minor_op_code,
-	YIPUnion *ip
+	const YIPUnion *ip
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop> 
@@ -976,21 +1021,20 @@ int YNetParseSetHost(YCNP_STD_INPUTS_PROTO)
 int YNetSendSetMixerDevice(
         YConnectionNumber con_num,
         int mixer_code,
-        Coefficient value1,
-        Coefficient value2
-)        
-{       
+        Coefficient value1, Coefficient value2
+)
+{
         /*   <u32_cs> <u16_majop> <u16_minop>
-         *   <u16_mixercode> <u16_value1> <u16_value2>
+         *   <u16_mixercode> <u32_value1> <u32_value2>
          */
-        const YDataLength this_seg_len = 8 + 6;
+        const YDataLength this_seg_len = 8 + 10;
 
         char buf[this_seg_len];
-        u_int16_t ival1, ival2;
+        u_int32_t ival1, ival2;
 
 
-        ival1 = (Coefficient)value1 * (Coefficient)((u_int16_t)-1);
-        ival2 = (Coefficient)value2 * (Coefficient)((u_int16_t)-1);
+        ival1 = (u_int32_t)rint(value1 * (Coefficient)((u_int32_t)-1));
+        ival2 = (u_int32_t)rint(value2 * (Coefficient)((u_int32_t)-1));
 
 
         *(u_int32_t *)(&buf[0]) = htonl((u_int32_t)this_seg_len);
@@ -999,8 +1043,8 @@ int YNetSendSetMixerDevice(
 
         *(u_int16_t *)(&buf[8]) = htons((u_int16_t)mixer_code);
 
-        *(u_int16_t *)(&buf[10]) = htons((u_int16_t)ival1);
-        *(u_int16_t *)(&buf[12]) = htons((u_int16_t)ival2);
+        *(u_int32_t *)(&buf[10]) = htonl((u_int32_t)ival1);
+        *(u_int32_t *)(&buf[14]) = htonl((u_int32_t)ival2);
 
         return(YNetSendToConnection(con_num, buf, this_seg_len));
 }
@@ -1012,11 +1056,11 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
         if(minor_op_code == YMixerChannelSet)
         {
             /*   <u32_cs> <u16_majop> <u16_minop>
-             *   <u16_mixercode> <u16_value1> <u16_value2>
+             *   <u16_mixercode> <u32_value1> <u32_value2>
              */
-            const YDataLength this_seg_len = 8 + 6;
+            const YDataLength this_seg_len = 8 + 10;
 	    int i, status, mixer_code;
-	    Coefficient value1, value2;
+	    Coefficient value[YMixerValues];
 	    YConnection **yc_ptr;
 
 
@@ -1025,32 +1069,27 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
 
             mixer_code = ntohs(*(u_int16_t *)(&buf[8]));
 
-            value1 =
-                (Coefficient)ntohs(*(u_int16_t *)(&buf[10])) /
-                (Coefficient)((u_int16_t)-1);
-            value2 =
-                (Coefficient)ntohs(*(u_int16_t *)(&buf[12])) /
-                (Coefficient)((u_int16_t)-1);
+	    if(0 < YMixerValues)
+                value[0] =
+                    (Coefficient)ntohl(*(u_int32_t *)(&buf[10])) /
+                    (Coefficient)((u_int32_t)-1);
+	    if(1 < YMixerValues)
+		value[1] =
+                    (Coefficient)ntohl(*(u_int32_t *)(&buf[14])) /
+                    (Coefficient)((u_int32_t)-1);
 
 	    /* Set mixer device. */
 	    status = -1;
 	    if(recorder != NULL)
 	    {
-		status = YMixerSet(
-		    recorder,
-		    mixer_code,
-		    value1,
-                    value2
-		);
+		status = YMixerSet(recorder, mixer_code, value);
 	    }
 	    if(status < 0)
 	    {
                 /* Error setting mixer values, notify error. */
                 YNetSendSetMixerDevice(
-                    con_num,
-                    mixer_code,
-                    0,
-                    0
+                    con_num, mixer_code,
+		    0.0, 0.0
                 );
 	    }
 	    else
@@ -1061,17 +1100,16 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
                     i++, yc_ptr++
                 )
                 {
-                    if(*yc_ptr == NULL)
+                    if((*yc_ptr) == NULL)
                         continue;
 
                     if((*yc_ptr)->socket < 0)
                         continue;
 
 		    YNetSendSetMixerDevice(
-		        i,
-                        mixer_code,
-		        value1,
-		        value2
+		        i, mixer_code,
+			(0 < YMixerValues) ? value[0] : 0.0,
+			(1 < YMixerValues) ? value[1] : 0.0
 		    );
 		}
 	    }
@@ -1083,7 +1121,7 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
              */
             const YDataLength this_seg_len = 8 + 2;
             int status, mixer_code;
-	    Coefficient value1, value2;
+	    Coefficient value[YMixerValues];
 
 
             if(chunk_length < this_seg_len)
@@ -1096,24 +1134,26 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
 	    if(recorder != NULL)
 	    {
 		status = YMixerGet(
-                    recorder,
-                    mixer_code,
-                    &value1,
-                    &value2
+                    recorder, mixer_code, value
 		);
 	    }
 	    if(status < 0)
 	    {
-		/* Error getting values. */
-                YNetSendSetMixerDevice(con_num, mixer_code, 0, 0);
+		/* Error getting values, respond with 0 channel
+		 * values.
+		 */
+                YNetSendSetMixerDevice(con_num, mixer_code, 0.0, 0.0);
 	    }
 	    else
 	    {
-                YNetSendSetMixerDevice(con_num, mixer_code, value1, value2);
+		YNetSendSetMixerDevice(
+		    con_num, mixer_code,
+		    (0 < YMixerValues) ? value[0] : 0.0,
+		    (1 < YMixerValues) ? value[1] : 0.0
+		);
 	    }
 	}
 
-
 	return(0);
 }
 
@@ -1124,22 +1164,25 @@ int YNetParseMixerDevice(YCNP_STD_INPUTS_PROTO)
 int YNetSendSoundObjectPlay(
 	YConnectionNumber con_num,
 	YID yid,
-	char *path,
+	const char *path,
 	YDataPosition pos,
 	YVolumeStruct *volume,
+	int sample_rate,
 	int repeats
 )
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
          *   <u32_yid> <u32_pos> <u16_lvol> <u16_rvol>
-         *   <u32_repeats>
+	 *   <u32_samplerate> <u32_repeats>
          *   <...path...>
          */
-        const YDataLength this_seg_len = 8 + 12 + 4 + YPathMax; 
+        const YDataLength this_seg_len = 8 + 12 + 8 + YPathMax; 
 
         char buf[this_seg_len];
         YDataLength path_len, actual_seg_len;
 
+	if(sample_rate < 0)
+	    sample_rate = 0;
 	if(repeats < 0)
 	    repeats = 0;
 
@@ -1152,11 +1195,12 @@ int YNetSendSoundObjectPlay(
         *(u_int16_t *)(&buf[16]) = htons((u_int16_t)volume->left);
         *(u_int16_t *)(&buf[18]) = htons((u_int16_t)volume->right);
 
-        *(u_int32_t *)(&buf[20]) = htonl((u_int32_t)repeats);
+	*(u_int32_t *)(&buf[20]) = htonl((u_int32_t)sample_rate);
+        *(u_int32_t *)(&buf[24]) = htonl((u_int32_t)repeats);
 
         path_len = MIN(strlen(path), YPathMax);
         strncpy(
-            &buf[24],
+            &buf[28],
             path,
             path_len
         );
@@ -1165,7 +1209,7 @@ int YNetSendSoundObjectPlay(
         /*   Since we have a string in this seg, we need to
          *   calculate the actual length.
          */
-        actual_seg_len = 8 + 12 + 4 + path_len;
+        actual_seg_len = 8 + 12 + 8 + path_len;
 
         /* Update segment header's chunk size! */
         *(u_int32_t *)(&buf[0]) = htonl((u_int32_t)actual_seg_len);
@@ -1179,15 +1223,16 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
 {
         /*   <u32_cs> <u16_majop> <u16_minop>
          *   <u32_yid> <u32_pos> <u16_lvol> <u16_rvol>
-         *   <u32_repeats>
+         *   <u32_sample_rate> <u32_repeats>
          *   <...path...>
          */
-        const YDataLength this_seg_len = 8 + 12 + 4 + YPathMax;
-        const YDataLength this_seg_len_base = 8 + 12 + 4;
+/*	const YDataLength this_seg_len = 8 + 12 + 8 + YPathMax; */
+	const YDataLength this_seg_len_base = 8 + 12 + 8;
 
 	YID yid;
 	YDataPosition pos;
 	u_int16_t lvol, rvol;
+	int sample_rate;
 	int repeats;
 
 	int path_len;
@@ -1203,9 +1248,10 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
         pos = ntohl(*(u_int32_t *)(&buf[12]));
         lvol = ntohs(*(u_int16_t *)(&buf[16]));
         rvol = ntohs(*(u_int16_t *)(&buf[18]));
-	repeats = ntohl(*(u_int32_t *)(&buf[20]));
+	sample_rate = ntohl(*(u_int32_t *)(&buf[20]));
+	repeats = ntohl(*(u_int32_t *)(&buf[24]));
 
-	path_len = (int)chunk_length - 24;
+	path_len = (int)chunk_length - this_seg_len_base;
 	if(path_len >= YPathMax)
 	    path_len = YPathMax - 1;
 
@@ -1213,7 +1259,7 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
 	{
 	    strncpy(
 		path,
-		&buf[24],
+		&buf[28],
 		path_len
 	    );
 	    path[path_len] = '\0';
@@ -1228,6 +1274,7 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
                 path,
                 pos,
                 &volume,
+		sample_rate,
                 repeats
             );
 	    return(0);
@@ -1244,6 +1291,7 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
 		path,
 		pos,
 		&volume,
+		sample_rate,
 		repeats
 	    );
 	    return(0);
@@ -1258,6 +1306,7 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
                 path,
                 pos,
                 &volume,
+		sample_rate,
                 repeats
             );
             return(0);
@@ -1272,6 +1321,7 @@ int YNetParseSoundObjectPlay(YCNP_STD_INPUTS_PROTO)
             yid,		/* YID. */
 	    pos,		/* Position. */
             &volume,
+	    sample_rate,
 	    repeats		/* 0 means repeat forever. */
 	);
 
@@ -1353,7 +1403,7 @@ int YNetParseSoundObjectKill(YCNP_STD_INPUTS_PROTO)
  */
 int YNetSendSoundObjectAttributes(
 	YConnectionNumber con_num,
-	char *path,
+	const char *path,
 	int format,
 	int sample_size,
 	int channels,   
@@ -1413,7 +1463,7 @@ int YNetParseSoundObjectAttributes(YCNP_STD_INPUTS_PROTO)
              *   <...path...>
              */
             const YDataLength base_seg_len = 8;
-	    const YDataLength this_seg_len = 8 + YPathMax;
+/*	    const YDataLength this_seg_len = 8 + YPathMax; */
 
 	    YDataLength path_len;
 	    char path[YPathMax];
@@ -1530,7 +1580,7 @@ int YNetParseShutdown(YCNP_STD_INPUTS_PROTO)
         reason = ntohs(*(u_int16_t *)(&buf[8]));
 
 
-/* Need to check if connection has permission to do so. */
+/* Check if connection has permission to shut server down? */
 
 	/*   Set runlevel to 1 to shutdown gently. Comfermation
 	 *   will be sent to all connections just before shutdown.
@@ -1716,7 +1766,7 @@ int YNetParseAudioStats(YCNP_STD_INPUTS_PROTO)
  */
 int YNetSendListAudioModes(
 	YConnectionNumber con_num,
-	char *audio_mode_name,
+	const char *audio_mode_name,
 	int sample_rate, int channels,
 	int sample_size, int fragment_size_bytes,
 	char direction, Boolean allow_fragmenting, int num_fragments
@@ -1836,6 +1886,155 @@ int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO)
         return(0);
 }
 
+/*
+ *	Sends sound object values.
+ */
+int YNetSendPlaySoundObjectValues(
+	YConnectionNumber con_num,
+	PlayStack *ps_ptr
+)
+{
+        /*   <u32_cs> <u16_majop> <u16_minop>
+         *   <u32_flags> <u32_yid>
+         *   <u32_pos> <u32_totalrepeats>
+         *   <u16_volleft> <u16_volright> <u32_sample_rate>
+         */
+	const YDataLength this_seg_len = 8 + 8 + 8 + 8;
+
+        char buf[this_seg_len];
+	u_int32_t flags;
+	u_int16_t volume_left, volume_right;
+
+
+	if(ps_ptr == NULL)
+	    return(-1);
+
+	/* Set flags to match client side. */
+	flags = (1 << 1) || (1 << 2) || (1 << 5) || (1 << 6) || (1 << 7);
+
+	/* Calculate volume. */
+	volume_left = ps_ptr->volume_left * (Coefficient)((u_int16_t)-1);
+        volume_right = ps_ptr->volume_right * (Coefficient)((u_int16_t)-1);
+
+
+        *(u_int32_t *)(&buf[0]) = htonl((u_int32_t)this_seg_len);
+        *(u_int16_t *)(&buf[4]) = htons((u_int16_t)YSoundObjectPlayValues);
+        *(u_int16_t *)(&buf[6]) = htons((u_int16_t)YSoundObjectPlayValuesSet);
+
+	*(u_int32_t *)(&buf[8]) = htonl((u_int32_t)flags);
+	*(u_int32_t *)(&buf[12]) = htonl((u_int32_t)ps_ptr->yid);
+
+        *(u_int32_t *)(&buf[16]) = htonl((u_int32_t)ps_ptr->position);
+        *(u_int32_t *)(&buf[20]) = htonl((u_int32_t)ps_ptr->total_repeats);
+
+        *(u_int16_t *)(&buf[24]) = htons((u_int16_t)volume_left);
+        *(u_int16_t *)(&buf[26]) = htons((u_int16_t)volume_right);
+
+        *(u_int32_t *)(&buf[28]) = htonl((u_int32_t)ps_ptr->applied_sample_rate);
+
+
+        return(YNetSendToConnection(con_num, buf, this_seg_len));
+}
+/*
+ *	Parses play sound object values.
+ */
+int YNetParsePlaySoundObjectValues(YCNP_STD_INPUTS_PROTO)
+{
+        if(minor_op_code == YSoundObjectPlayValuesGet)
+        {
+            /* Parse list audio modes get. */
+
+            /*   <u32_cs> <u16_majop> <u16_minop>
+             */
+            const YDataLength this_seg_len = 8;
+
+            if(chunk_length < this_seg_len)
+                return(-1);
+	}
+        else if(minor_op_code == YSoundObjectPlayValuesSet)
+        {
+            /* Parse list audio modes get. */
+
+            /*   <u32_cs> <u16_majop> <u16_minop>
+             *   <u32_flags> <u32_yid>
+             *   <u32_pos> <u32_totalrepeats>
+             *   <u16_volleft> <u16_volright> <u32_sample_rate>
+             */
+            const YDataLength this_seg_len = 8 + 8 + 8 + 8;
+
+	    int i;
+	    u_int32_t flags;
+	    YID yid;
+	    u_int32_t position;
+	    u_int32_t total_repeats;
+	    u_int16_t volume_left, volume_right;
+	    u_int32_t sample_rate;
+	    PlayStack **ptr, *ps_ptr;
+
+
+            if(chunk_length < this_seg_len)
+                return(-1);
+
+            flags = ntohl(*(u_int32_t *)(&buf[8]));
+	    yid = ntohl(*(u_int32_t *)(&buf[12]));
+
+            position = ntohl(*(u_int32_t *)(&buf[16]));
+            total_repeats = ntohl(*(u_int32_t *)(&buf[20]));
+
+	    volume_left = ntohs(*(u_int16_t *)(&buf[24]));
+            volume_right = ntohs(*(u_int16_t *)(&buf[26]));
+
+            sample_rate = ntohl(*(u_int32_t *)(&buf[28]));
+
+	    /* Check if yid exists. */
+            for(i = 0, ptr = playstack;
+                i < total_playstacks;
+                i++, ptr++
+            )
+            {
+		ps_ptr = *ptr;
+                if(ps_ptr == NULL)
+                    continue;
+
+                if(ps_ptr->owner != con_num)
+                    continue;
+
+                if(ps_ptr->yid == YIDNULL)
+                    continue;
+ 
+                if(ps_ptr->yid == yid)
+		{
+		    /* Set new values to play stack. */
+
+		    /* Change position? */
+		    if(flags & (1 << 2))
+			ps_ptr->position = (YDataPosition)position;
+
+		    /* Change total repeats? */
+		    if(flags & (1 << 5))
+			ps_ptr->total_repeats = (int)total_repeats;
+
+                    /* Change volume? */
+                    if(flags & (1 << 6))
+		    {
+                        ps_ptr->volume_left = (double)volume_left /
+			    (double)((u_int16_t)-1);
+			ps_ptr->volume_right = (double)volume_right /
+                            (double)((u_int16_t)-1);
+		    }
+
+		    /* Change sample size? */
+		    if(flags & (1 << 7))
+			ps_ptr->applied_sample_rate = (int)sample_rate;
+
+		    /* Send response of values set. */
+		    YNetSendPlaySoundObjectValues(con_num, ps_ptr);
+		}
+	    }
+        }
+
+        return(0);
+}
 
 
 /*
@@ -1843,7 +2042,7 @@ int YNetParseListAudioModes(YCNP_STD_INPUTS_PROTO)
  */
 int YNetParse(
         YConnectionNumber con_num,
-        u_int8_t *buf,
+	const u_int8_t *buf,
         u_int32_t chunk_length,
         u_int16_t major_op_code,
         u_int16_t minor_op_code
@@ -1919,6 +2118,10 @@ int YNetParse(
             status = YNetParseListAudioModes(YCNP_STD_INPUTS);
             break;
 
+	  case YSoundObjectPlayValues:
+	    status = YNetParsePlaySoundObjectValues(YCNP_STD_INPUTS);
+            break;
+
 
           default:
 	    YNetPrintError(
@@ -2073,7 +2276,9 @@ int YNetRecv(YConnectionNumber con_num)
              *   <4 bytes, chunk length> <2 bytes, major op code>
              *   <2 bytes, minor op code>
              */
-            chunk_length = ntohl(*((u_int32_t *)(&con->buf[0])));
+            chunk_length = (u_int32_t)MAX(
+		(long)ntohl(*((u_int32_t *)(&con->buf[0]))), (long)0
+	    );
             major_op_code = ntohs(*((u_int16_t *)(&con->buf[4])));
             minor_op_code = ntohs(*((u_int16_t *)(&con->buf[6])));
 
