@@ -3,14 +3,13 @@
 
 
 #include <stdio.h>
-#include <db.h>
+#include <limits.h>
 #include <sys/types.h>
 
-#include "../include/Y.h"
-
-#ifdef __linux__
-# include <linux/limits.h>
-#endif	/* __linux__ */
+/* Include header file for libY2 that comes with this distribution,
+ * this is to define major and minor op codes for network parsing.
+ */
+#include "../include/Y2/Y.h"
 
 
 
